(herald "Four Party Needham-Schroeder-Lowe Protocol")

(comment "CPSA 2.3.0")
(comment "All input read from nsl4cm1.lsp")

(defprotocol nsl4cm basic
  (defrole init
    (vars (a b c d name) (na nb nc nd text))
    (trace (send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b)))))
  (defrole resp1
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a c d na (pubk b)))
      (send (enc a b d na nb (pubk c))) (recv (enc nb nc nd (pubk b)))
      (send (enc nc nd (pubk c)))))
  (defrole resp2
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))) (recv (enc nc nd (pubk c)))
      (send (enc nd (pubk d)))))
  (defrole resp3
    (vars (a b c d name) (na nb nc nd text))
    (trace (recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a)))
      (recv (enc nd (pubk d))))))

(defskeleton nsl4cm
  (vars (na nb nc nd text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b)))))
  (label 0)
  (unrealized (0 1))
  (origs (na (0 0)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (0 1)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 1)
  (parent 0)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (0 1)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 2)
  (parent 1)
  (unrealized (0 1) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 3 1 resp1 2) na (2 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 3)
  (parent 2)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (2 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 4)
  (parent 2)
  (seen 3)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 3 1 resp1 2) na (2 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 5)
  (parent 2)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 6)
  (parent 3)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 4 1 resp1 2) na (2 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 7)
  (parent 4)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (2 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 8)
  (parent 4)
  (unrealized (0 1) (2 0) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 9)
  (parent 5)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 10)
  (parent 5)
  (seen 13)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 4 1 resp1 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 11)
  (parent 6)
  (seen 14)
  (unrealized (0 1) (3 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 12)
  (parent 6)
  (seen 26)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 4 1 resp1 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 13)
  (parent 6)
  (seen 25 28)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 4 2 resp2 2) na (3 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 14)
  (parent 6)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 15)
  (parent 7)
  (seen 35)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 16)
  (parent 7)
  (seen 40)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 17)
  (parent 8)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 3 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 18)
  (parent 8)
  (seen 45)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 19)
  (parent 8)
  (seen 18)
  (unrealized (0 1) (2 0) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 20)
  (parent 8)
  (seen 52)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 5 3 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 21)
  (parent 8)
  (seen 55)
  (unrealized (0 1) (2 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 22)
  (parent 9)
  (seen 63)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 23)
  (parent 9)
  (seen 24 68)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (3 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 24)
  (parent 10)
  (seen 28 31)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 4 2 resp2 2) nb-0 (3 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 25)
  (parent 10)
  (seen 70)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 4 2 resp2 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 26)
  (parent 11)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 27)
  (parent 11)
  (unrealized (0 1) (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (3 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 28)
  (parent 12)
  (seen 31 72)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 29)
  (parent 12)
  (unrealized (0 1) (3 0) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) na (3 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 30)
  (parent 12)
  (seen 33 34)
  (unrealized (0 1))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 4 2 resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 31)
  (parent 13)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 32)
  (parent 13)
  (seen 78)
  (unrealized (0 1) (3 0) (4 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 33)
  (parent 14)
  (seen 75)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 34)
  (parent 14)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 35)
  (parent 15)
  (seen 5 39 47)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 36)
  (parent 15)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 37)
  (parent 15)
  (seen 36 47 100)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 38)
  (parent 15)
  (seen 9 10)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 39)
  (parent 15)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 40)
  (parent 16)
  (seen 44)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 41)
  (parent 16)
  (seen 31 44 112)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 42)
  (parent 16)
  (seen 113 114)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 43)
  (parent 16)
  (seen 28 31 32)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 44)
  (parent 16)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 45)
  (parent 17)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 3 resp1 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 46)
  (parent 17)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 5 3 resp1 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 47)
  (parent 17)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 48)
  (parent 17)
  (unrealized (0 1) (4 0) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 49)
  (parent 18)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 50)
  (parent 18)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 51)
  (parent 18)
  (unrealized (0 1) (2 0) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (4 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 52)
  (parent 19)
  (seen 56)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (4 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 53)
  (parent 19)
  (seen 47 145)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 54)
  (parent 19)
  (unrealized (0 1) (2 0) (4 0) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0)) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 55)
  (parent 20)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 56)
  (parent 20)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 57)
  (parent 20)
  (seen 160 161 162)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 58)
  (parent 20)
  (seen 133)
  (unrealized (0 1) (4 0) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 5 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 59)
  (parent 21)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 60)
  (parent 21)
  (seen 59)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 61)
  (parent 21)
  (seen 160 161 162)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 62)
  (parent 21)
  (seen 143)
  (unrealized (0 1) (2 0) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 63)
  (parent 22)
  (seen 5 21 38 67)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 64)
  (parent 22)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 65)
  (parent 22)
  (seen 21 64 179)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 66)
  (parent 22)
  (seen 187)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 67)
  (parent 22)
  (seen 185)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 68)
  (parent 23)
  (seen 28 70 91)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 69)
  (parent 23)
  (seen 31)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (4 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 70)
  (parent 23)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 71)
  (parent 24)
  (seen 86)
  (unrealized (0 1) (3 0) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-1 (3 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 72)
  (parent 24)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nc nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 73)
  (parent 25)
  (seen 90)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 74)
  (parent 25)
  (seen 96)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a)))))
  (label 75)
  (parent 26)
  (unrealized)
  (shape)
  (maps ((0) ((a a) (b b) (c c) (d d) (na na) (nb nb) (nc nc) (nd nd))))
  (origs (na (0 0))))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 76)
  (parent 27)
  (seen 26)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 77)
  (parent 27)
  (seen 76)
  (unrealized (0 1) (3 0) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 78)
  (parent 27)
  (seen 31)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 79)
  (parent 27)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 80)
  (parent 28)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 81)
  (parent 28)
  (seen 92 217)
  (unrealized (0 1) (3 0) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 82)
  (parent 28)
  (seen 86 203 224)
  (unrealized (0 1) (3 0) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 83)
  (parent 29)
  (seen 76 88)
  (unrealized (0 1) (3 0) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 84)
  (parent 29)
  (seen 226)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 85)
  (parent 29)
  (seen 84 230)
  (unrealized (0 1) (3 0) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 86)
  (parent 29)
  (seen 78 203 237)
  (unrealized (0 1) (3 0) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 87)
  (parent 29)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 88)
  (parent 29)
  (seen 79)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 89)
  (parent 30)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a)))))
  (label 90)
  (parent 31)
  (seen 75)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 91)
  (parent 31)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 92)
  (parent 32)
  (seen 213 224)
  (unrealized (0 1) (3 0) (4 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 93)
  (parent 32)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 1 resp1 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 94)
  (parent 34)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 95)
  (parent 34)
  (seen 94 97)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) nc-1 (4 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 96)
  (parent 34)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 97)
  (parent 34)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 5 3 resp3 2) nc-1 (4 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d)))))
  (label 98)
  (parent 34)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 99)
  (parent 35)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 100)
  (parent 35)
  (seen 7 47 99 108)
  (unrealized (0 1) (4 0))
  (comment "9 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 101)
  (parent 35)
  (seen 9 10)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 102)
  (parent 35)
  (seen 290)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 103)
  (parent 36)
  (seen 295)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 104)
  (parent 36)
  (seen 301)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (4 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 105)
  (parent 37)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 106)
  (parent 37)
  (seen 15 16)
  (unrealized (0 1))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 107)
  (parent 37)
  (seen 310)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 108)
  (parent 37)
  (seen 109 110 111)
  (unrealized (0 1))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 109)
  (parent 39)
  (seen 318)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 110)
  (parent 39)
  (seen 324)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 111)
  (parent 39)
  (seen 330)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 112)
  (parent 40)
  (seen 31 44 114)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 113)
  (parent 40)
  (seen 114 119)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 114)
  (parent 40)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 115)
  (parent 40)
  (seen 342)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 116)
  (parent 41)
  (seen 347)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 117)
  (parent 42)
  (seen 31 80 81 82)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 118)
  (parent 42)
  (seen 353)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 119)
  (parent 42)
  (seen 121 122 123)
  (unrealized (0 1))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 120)
  (parent 43)
  (seen 70 73 74)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nc nb nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b d nc nc (pubk c))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 121)
  (parent 44)
  (seen 73)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 122)
  (parent 44)
  (seen 361)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 123)
  (parent 44)
  (seen 299 328 364 367)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 124)
  (parent 45)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a)))))
  (label 125)
  (parent 45)
  (seen 97)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 126)
  (parent 46)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 127)
  (parent 46)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 128)
  (parent 47)
  (seen 334 383)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 129)
  (parent 47)
  (seen 328 390)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 130)
  (parent 48)
  (unrealized (0 1) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 131)
  (parent 48)
  (seen 401)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 132)
  (parent 48)
  (seen 130)
  (unrealized (0 1) (4 0) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 133)
  (parent 48)
  (seen 167)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 134)
  (parent 48)
  (seen 409 411)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 135)
  (parent 49)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-1 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 136)
  (parent 49)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 137)
  (parent 50)
  (seen 421)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 138)
  (parent 50)
  (seen 428)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 139)
  (parent 51)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 140)
  (parent 51)
  (seen 144 442)
  (unrealized (0 1) (2 0) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 141)
  (parent 51)
  (seen 448)
  (unrealized (0 1) (2 0) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 142)
  (parent 51)
  (seen 447 450)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 143)
  (parent 51)
  (seen 177 443 451)
  (unrealized (0 1) (2 0) (5 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 144)
  (parent 51)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 145)
  (parent 52)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 146)
  (parent 52)
  (seen 463 464 465)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 147)
  (parent 52)
  (seen 165 166 413)
  (unrealized (0 1) (4 0) (6 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 148)
  (parent 52)
  (seen 157 467)
  (unrealized (0 1) (4 0) (6 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 149)
  (parent 53)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 150)
  (parent 53)
  (seen 463 464 465)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 151)
  (parent 53)
  (seen 476)
  (unrealized (0 1) (2 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 152)
  (parent 53)
  (seen 483)
  (unrealized (0 1) (2 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 153)
  (parent 54)
  (seen 130)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 154)
  (parent 54)
  (seen 489)
  (unrealized (0 1) (2 0) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 155)
  (parent 54)
  (seen 469 494)
  (unrealized (0 1) (2 0) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 157)
  (parent 54)
  (seen 133 496)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 158)
  (parent 54)
  (seen 491)
  (unrealized (0 1) (2 0) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 159)
  (parent 54)
  (seen 507)
  (unrealized (0 1) (2 0) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 160)
  (parent 55)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 161)
  (parent 55)
  (seen 372)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 162)
  (parent 55)
  (seen 208 377)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 163)
  (parent 56)
  (seen 382 395)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 164)
  (parent 56)
  (seen 389 399)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 165)
  (parent 58)
  (seen 167 406)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 166)
  (parent 58)
  (seen 408 467)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 167)
  (parent 58)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 168)
  (parent 59)
  (seen 422)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 169)
  (parent 59)
  (seen 429)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (2 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 170)
  (parent 60)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 171)
  (parent 60)
  (seen 550 551 552)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 172)
  (parent 60)
  (seen 554)
  (unrealized (0 1) (2 0) (6 0))
  (comment "8 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 173)
  (parent 60)
  (seen 174 175 560)
  (unrealized (0 1) (2 0) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 174)
  (parent 62)
  (seen 443)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 175)
  (parent 62)
  (seen 451 457)
  (unrealized (0 1) (2 0) (5 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 176)
  (parent 62)
  (seen 564 567)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 177)
  (parent 62)
  (seen 563)
  (unrealized (0 1) (2 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 178)
  (parent 63)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 179)
  (parent 63)
  (seen 7 21 60 178 185)
  (unrealized (0 1) (4 0))
  (comment "9 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 180)
  (parent 63)
  (seen 21)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 181)
  (parent 64)
  (seen 585)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 182)
  (parent 64)
  (seen 591)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 183)
  (parent 65)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 184)
  (parent 65)
  (seen 559 600)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-2 (4 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 185)
  (parent 65)
  (seen 189)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 186)
  (parent 66)
  (seen 612)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 187)
  (parent 66)
  (seen 588 594)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nc-0)) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 188)
  (parent 67)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 3 resp1 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 189)
  (parent 67)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp1 2) nc-1 (4 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 190)
  (parent 67)
  (seen 616 617 618)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 191)
  (parent 67)
  (seen 584 622)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 192)
  (parent 67)
  (seen 177 626)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 193)
  (parent 68)
  (seen 92 224 268 630)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 194)
  (parent 69)
  (seen 31 80)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 195)
  (parent 69)
  (seen 78 202)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 196)
  (parent 69)
  (seen 70 73 74)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nc nb nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b d nc nc (pubk c))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 197)
  (parent 70)
  (seen 73)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 198)
  (parent 70)
  (seen 207 277 614)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 199)
  (parent 70)
  (seen 589 614 638)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 200)
  (parent 71)
  (seen 226 245 248)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 201)
  (parent 71)
  (seen 200 237 243)
  (unrealized (0 1) (3 0) (5 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 202)
  (parent 71)
  (seen 245 248)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 203)
  (parent 71)
  (seen 93 646)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nc nd nb text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b d nc nc (pubk c))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c)))))
  (label 204)
  (parent 72)
  (seen 73)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 205)
  (parent 72)
  (seen 654)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 206)
  (parent 72)
  (seen 258 657)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 207)
  (parent 74)
  (seen 208 273 614)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nc-0 (4 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 208)
  (parent 74)
  (seen 636)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 5 3 resp3 2) nc-1 (4 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d)))))
  (label 209)
  (parent 74)
  (seen 637)
  (unrealized (0 1))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 210)
  (parent 76)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 211)
  (parent 76)
  (unrealized (0 1) (3 0) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 212)
  (parent 77)
  (seen 234)
  (unrealized (0 1) (3 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 213)
  (parent 77)
  (seen 31 114)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 214)
  (parent 77)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 215)
  (parent 77)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 216)
  (parent 77)
  (seen 220 221)
  (unrealized (0 1))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 217)
  (parent 78)
  (seen 31 80)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 218)
  (parent 78)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 219)
  (parent 78)
  (seen 676)
  (unrealized (0 1) (3 0) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d)))))
  (label 220)
  (parent 79)
  (seen 33)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 221)
  (parent 79)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 222)
  (parent 80)
  (seen 90)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 223)
  (parent 80)
  (seen 662 710)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 224)
  (parent 81)
  (seen 248 648)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 225)
  (parent 81)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 226)
  (parent 82)
  (seen 248 642 715)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 227)
  (parent 82)
  (seen 226 237 243 646)
  (unrealized (0 1) (3 0) (5 0))
  (comment "8 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 228)
  (parent 83)
  (seen 26)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 229)
  (parent 83)
  (seen 31)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 230)
  (parent 83)
  (seen 214)
  (unrealized (0 1) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 231)
  (parent 83)
  (unrealized (0 1) (3 0) (5 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d)))))
  (label 232)
  (parent 84)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 233)
  (parent 84)
  (seen 234 235 679)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d)))))
  (label 234)
  (parent 84)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 235)
  (parent 84)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 236)
  (parent 84)
  (unrealized (0 1) (3 0) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 237)
  (parent 85)
  (seen 213 241 243 646)
  (unrealized (0 1) (3 0) (5 0))
  (comment "9 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 238)
  (parent 85)
  (unrealized (0 1) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 240)
  (parent 85)
  (seen 79 251 253 254)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 241)
  (parent 86)
  (seen 31)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 242)
  (parent 86)
  (seen 31 217 219)
  (unrealized (0 1) (3 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 243)
  (parent 86)
  (seen 264 761)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 244)
  (parent 86)
  (seen 741)
  (unrealized (0 1) (3 0) (5 0) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 245)
  (parent 87)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 246)
  (parent 87)
  (seen 248 249 715)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 247)
  (parent 87)
  (seen 788)
  (unrealized (0 1) (3 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 248)
  (parent 87)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 249)
  (parent 87)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 250)
  (parent 87)
  (seen 757)
  (unrealized (0 1) (3 0) (6 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 251)
  (parent 88)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp1 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d)))))
  (label 252)
  (parent 88)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 253)
  (parent 88)
  (seen 743)
  (unrealized (0 1) (5 0) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 254)
  (parent 88)
  (unrealized (0 1) (5 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 255)
  (parent 89)
  (seen 94 260 261)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 256)
  (parent 89)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 257)
  (parent 89)
  (seen 256 260)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 258)
  (parent 89)
  (seen 96 662 663 831)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d)))))
  (label 259)
  (parent 89)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 260)
  (parent 89)
  (seen 97 261 446)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-0 (pubk d)))))
  (label 261)
  (parent 89)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 262)
  (parent 91)
  (seen 90)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 263)
  (parent 91)
  (seen 640 849)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 264)
  (parent 92)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 265)
  (parent 92)
  (seen 691)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 266)
  (parent 92)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d)))))
  (label 267)
  (parent 93)
  (seen 73)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 268)
  (parent 93)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 269)
  (parent 93)
  (seen 706)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d)))))
  (label 270)
  (parent 94)
  (seen 33)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 271)
  (parent 94)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nb-0)) nc-0 (4 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 272)
  (parent 95)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (4 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 273)
  (parent 95)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (4 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 274)
  (parent 95)
  (unrealized (0 1) (4 0) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (4 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c)))))
  (label 275)
  (parent 95)
  (seen 281 282 283)
  (unrealized (0 1))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nd nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nb) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nb) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nb) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nb (pubk d)) (enc a b c nb nb nc (pubk d))
    (enc a b d nb nb (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nb nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nb nd (pubk a)))
      (send (enc nb nb nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nb (pubk d))))
    ((recv (enc a b c nb nb nb (pubk d)))
      (send (enc b c d nb nb nb nd (pubk a))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 276)
  (parent 96)
  (seen 73)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 277)
  (parent 96)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 278)
  (parent 96)
  (seen 873)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d)))))
  (label 279)
  (parent 97)
  (seen 33)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 280)
  (parent 97)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nd nc text) (a b c d name))
  (defstrand init 3 (na nd) (nb nb) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd) (nb nb) (nc nd) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd) (nb nb) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nd) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nd) (nd-0 nd)) nd
    (0 1) (enc a b c nd nb nd (pubk d)) (enc a b c nd nb nc (pubk d))
    (enc a b d nd nb (pubk c)) (enc a c d nd (pubk b))
    (enc b c d nd nb nd nd (pubk a)))
  (traces
    ((send (enc a c d nd (pubk b)))
      (recv (enc b c d nd nb nd nd (pubk a)))
      (send (enc nb nd nd (pubk b))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nb (pubk c))))
    ((recv (enc a b d nd nb (pubk c)))
      (send (enc a b c nd nb nd (pubk d))))
    ((recv (enc a b c nd nb nd (pubk d)))
      (send (enc b c d nd nb nd nd (pubk a))))
    ((recv (enc a b d nd nb (pubk c)))
      (send (enc a b c nd nb nc (pubk d)))))
  (label 281)
  (parent 98)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nd-0 (pubk d))
    (enc a b c nd-0 nb-0 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a c d nd-0 (pubk b)) (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-1 (pubk a)))))
  (label 282)
  (parent 98)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nd-0 (pubk d))
    (enc a b c nd-0 nb-0 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a c d nd-0 (pubk b)) (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 283)
  (parent 98)
  (seen 908)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 284)
  (parent 99)
  (seen 915)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 285)
  (parent 99)
  (seen 918 921)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 286)
  (parent 100)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 287)
  (parent 100)
  (seen 15 16 309)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 288)
  (parent 100)
  (seen 291 292 928)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 289)
  (parent 100)
  (seen 310 316 930)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 290)
  (parent 102)
  (seen 47 55 293 928)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 291)
  (parent 102)
  (seen 47 293 294 411)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 292)
  (parent 102)
  (seen 930 940)
  (unrealized (0 1) (4 0) (5 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 293)
  (parent 102)
  (seen 160 161 162)
  (unrealized (0 1))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 294)
  (parent 102)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 295)
  (parent 103)
  (seen 299 345)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 296)
  (parent 103)
  (seen 31 299 300 348 949)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 297)
  (parent 103)
  (seen 348 950 951)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 298)
  (parent 103)
  (seen 28 31 32 78 364)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 299)
  (parent 103)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 300)
  (parent 103)
  (seen 953)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 301)
  (parent 104)
  (seen 229 305 825)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 302)
  (parent 104)
  (seen 78 305 306 968)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 303)
  (parent 104)
  (seen 229 969 972)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 304)
  (parent 104)
  (seen 28 31 32 78)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 305)
  (parent 104)
  (seen 25)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 306)
  (parent 104)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 307)
  (parent 105)
  (seen 989)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 308)
  (parent 105)
  (seen 997)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (1 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 309)
  (parent 106)
  (seen 273 1005)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 310)
  (parent 107)
  (seen 290 316 492)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 311)
  (parent 107)
  (seen 133 316 1010)
  (unrealized (0 1) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 312)
  (parent 107)
  (seen 933)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 314)
  (parent 107)
  (seen 52 55 56 58 133)
  (unrealized (0 1) (6 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 315)
  (parent 107)
  (seen 1022)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 316)
  (parent 107)
  (seen 294)
  (unrealized (0 1) (6 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a b d nc-0 nb-1 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 317)
  (parent 108)
  (seen 1039)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 318)
  (parent 109)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 319)
  (parent 109)
  (seen 188 321 322 323 1050)
  (unrealized (0 1) (5 0))
  (comment "9 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 320)
  (parent 109)
  (seen 322 1051 1053 1054)
  (unrealized (0 1) (5 0))
  (comment "10 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-2 (5 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 321)
  (parent 109)
  (seen 616 617 618)
  (unrealized (0 1))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 322)
  (parent 109)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-2 (5 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 323)
  (parent 109)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 324)
  (parent 110)
  (seen 328 329)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 325)
  (parent 110)
  (seen 208 328 329 351 1066)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 326)
  (parent 110)
  (seen 328 351 1067)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 327)
  (parent 110)
  (seen 93 208)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 328)
  (parent 110)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 329)
  (parent 110)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 330)
  (parent 111)
  (seen 334 335)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 331)
  (parent 111)
  (seen 334 335 522 1084)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 332)
  (parent 111)
  (seen 334 1085 1088)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 333)
  (parent 111)
  (seen 522 1076)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 334)
  (parent 111)
  (seen 335 1086 1088)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 335)
  (parent 111)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 336)
  (parent 112)
  (seen 343 345 346 1104)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 337)
  (parent 112)
  (seen 345 347 351)
  (unrealized (0 1) (4 0) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 338)
  (parent 113)
  (seen 343 344 1108)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 339)
  (parent 113)
  (seen 353 359 1110)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a b d nb nb-0 (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a)))))
  (label 340)
  (parent 114)
  (seen 90)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 341)
  (parent 114)
  (seen 328 951 1116)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 342)
  (parent 115)
  (seen 114 1104 1108)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 343)
  (parent 115)
  (seen 264 345 346 1123)
  (unrealized (0 1) (4 0) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 344)
  (parent 115)
  (seen 1110 1125)
  (unrealized (0 1) (4 0) (5 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 345)
  (parent 115)
  (seen 31 217 219)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 346)
  (parent 115)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 347)
  (parent 116)
  (seen 78 351)
  (unrealized (0 1) (4 0) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 348)
  (parent 116)
  (seen 31 1106)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 349)
  (parent 116)
  (seen 348 1135 1137)
  (unrealized (0 1) (4 0) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 350)
  (parent 116)
  (seen 31 217 219 1130)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 351)
  (parent 116)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 352)
  (parent 117)
  (seen 204 205 206)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 353)
  (parent 118)
  (seen 342 359)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 354)
  (parent 118)
  (seen 243 348 359 1149)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 355)
  (parent 118)
  (seen 1113)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 357)
  (parent 118)
  (seen 78 237 241 243 244)
  (unrealized (0 1) (4 0) (6 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 358)
  (parent 118)
  (seen 1158 1161)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 359)
  (parent 118)
  (seen 346 351)
  (unrealized (0 1) (6 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a b d nc-0 nb-1 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 360)
  (parent 119)
  (seen 1175)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 361)
  (parent 122)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 362)
  (parent 122)
  (seen 208 364 365 366 1186)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 363)
  (parent 122)
  (seen 365 1188 1189)
  (unrealized (0 1) (5 0))
  (comment "9 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 364)
  (parent 122)
  (seen 276 277 278)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 365)
  (parent 122)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 4 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d)))))
  (label 366)
  (parent 122)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 367)
  (parent 123)
  (seen 96 328 369 919)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 368)
  (parent 123)
  (seen 299 328 995 1190 1200)
  (unrealized (0 1) (5 0))
  (comment "9 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 4 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d)))))
  (label 369)
  (parent 123)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 370)
  (parent 124)
  (seen 373 374)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 371)
  (parent 124)
  (seen 1209 1210)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 372)
  (parent 124)
  (seen 521 522 1212)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 373)
  (parent 124)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 5 3 resp2 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 374)
  (parent 124)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a)))))
  (label 375)
  (parent 125)
  (seen 97 214 378)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 376)
  (parent 125)
  (seen 214 1226)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 1 resp1 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 377)
  (parent 125)
  (seen 208 264 524 1228)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) na (4 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 378)
  (parent 125)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 379)
  (parent 126)
  (seen 384 385 1210)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 380)
  (parent 126)
  (seen 384 385 1209 1236)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 381)
  (parent 126)
  (seen 1238 1241)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 382)
  (parent 126)
  (seen 527 528 1240 1243)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 383)
  (parent 126)
  (seen 334 396 1237 1244)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 384)
  (parent 126)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 385)
  (parent 126)
  (seen 1221 1222)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 386)
  (parent 127)
  (seen 391 392 1226)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 387)
  (parent 127)
  (seen 214 391 392 1262)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 388)
  (parent 127)
  (seen 1264 1267)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 389)
  (parent 127)
  (seen 531 532 1266 1269)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 390)
  (parent 127)
  (seen 328 400 1263 1270)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 391)
  (parent 127)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 392)
  (parent 127)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 393)
  (parent 128)
  (seen 334 396 1217 1237)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 394)
  (parent 128)
  (seen 1244 1252 1286)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 395)
  (parent 128)
  (seen 1212 1216 1217 1218)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 396)
  (parent 128)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 397)
  (parent 129)
  (seen 328 400 1137 1232 1263)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 398)
  (parent 129)
  (seen 1137 1270 1278)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 399)
  (parent 129)
  (seen 264 1228 1232 1233)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 400)
  (parent 129)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-1 nb-0)) na (4 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 401)
  (parent 130)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (4 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 402)
  (parent 130)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (4 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 403)
  (parent 130)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-0 (pubk a)))))
  (label 404)
  (parent 130)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 405)
  (parent 131)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 406)
  (parent 131)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 407)
  (parent 131)
  (unrealized (0 1) (5 0) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 408)
  (parent 132)
  (seen 406 534)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 409)
  (parent 132)
  (seen 415 1333)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 411)
  (parent 133)
  (seen 536 537 538 539)
  (unrealized (0 1))
  (comment "4 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 412)
  (parent 133)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 413)
  (parent 133)
  (seen 412 466)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 414)
  (parent 133)
  (seen 1319)
  (unrealized (0 1) (4 0) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 415)
  (parent 134)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 416)
  (parent 134)
  (seen 536 537 538 539)
  (unrealized (0 1))
  (comment "4 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d)))))
  (label 417)
  (parent 134)
  (seen 1332)
  (unrealized (0 1) (5 0) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 418)
  (parent 135)
  (seen 423 1210)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 419)
  (parent 135)
  (seen 423 424 1209 1356)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 420)
  (parent 135)
  (seen 1358 1362)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 421)
  (parent 135)
  (seen 435 1361 1364)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 422)
  (parent 135)
  (seen 543 544 1357 1365)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 423)
  (parent 135)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 424)
  (parent 135)
  (seen 1221 1222 1360 1372)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 425)
  (parent 136)
  (seen 430 735 841 1226)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 426)
  (parent 136)
  (seen 214 430 431 1382)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 427)
  (parent 136)
  (seen 735 1386)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 428)
  (parent 136)
  (seen 440 736 842 1385 1388)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 429)
  (parent 136)
  (seen 548 549 780 1383 1389)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 430)
  (parent 136)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 431)
  (parent 136)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 432)
  (parent 137)
  (seen 435 436 1216 1361)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 433)
  (parent 137)
  (seen 1364 1369 1403)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 434)
  (parent 137)
  (seen 1212 1216 1217 1218)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 435)
  (parent 137)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 436)
  (parent 137)
  (seen 1094 1372)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 437)
  (parent 138)
  (seen 264 440 441 1385)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 438)
  (parent 138)
  (seen 736 1388 1418)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 439)
  (parent 138)
  (seen 264 1228 1232 1233)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 440)
  (parent 138)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 441)
  (parent 138)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 442)
  (parent 139)
  (seen 444 446)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 443)
  (parent 139)
  (seen 564 566)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a)))))
  (label 444)
  (parent 139)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 445)
  (parent 139)
  (unrealized (0 1) (5 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 446)
  (parent 139)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 447)
  (parent 140)
  (seen 454 456)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 448)
  (parent 140)
  (seen 444)
  (unrealized (0 1) (2 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 449)
  (parent 140)
  (unrealized (0 1) (2 0) (5 0) (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 450)
  (parent 141)
  (seen 1450)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 451)
  (parent 141)
  (seen 457 569)
  (unrealized (0 1) (2 0) (5 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c)))))
  (label 453)
  (parent 141)
  (seen 446 459 460 462)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 454)
  (parent 142)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 455)
  (parent 142)
  (seen 1443)
  (unrealized (0 1) (5 0) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 456)
  (parent 142)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 457)
  (parent 143)
  (seen 564)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 458)
  (parent 143)
  (seen 1458)
  (unrealized (0 1) (2 0) (5 0) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (contracted (nb-1 nb-0)) nc-1 (2 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 459)
  (parent 144)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 460)
  (parent 144)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nb-0 nd-0 (pubk a)))))
  (label 461)
  (parent 144)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 462)
  (parent 144)
  (unrealized (0 1) (2 0) (6 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 463)
  (parent 145)
  (seen 322 1501)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 464)
  (parent 145)
  (seen 395 1506)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 465)
  (parent 145)
  (seen 365 399 1511)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 466)
  (parent 147)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 467)
  (parent 147)
  (seen 411 412 413 414)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 468)
  (parent 148)
  (seen 496)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 469)
  (parent 148)
  (seen 1520)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 471)
  (parent 148)
  (seen 411)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 472)
  (parent 149)
  (seen 1534)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 473)
  (parent 149)
  (seen 1542)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 474)
  (parent 149)
  (seen 1550)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 8 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 475)
  (parent 151)
  (seen 406 1558)
  (unrealized (0 1) (6 0))
  (comment "8 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 476)
  (parent 151)
  (seen 21 481 497 1558)
  (unrealized (0 1) (2 0) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 477)
  (parent 151)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 479)
  (parent 151)
  (seen 406)
  (unrealized (0 1) (6 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 480)
  (parent 151)
  (seen 55 59 60 62)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 481)
  (parent 151)
  (seen 1033 1051)
  (unrealized (0 1) (2 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 482)
  (parent 152)
  (seen 1585)
  (unrealized (0 1) (6 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 483)
  (parent 152)
  (seen 488 1585)
  (unrealized (0 1) (2 0) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 484)
  (parent 152)
  (seen 457 488 1586 1591)
  (unrealized (0 1) (2 0) (6 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 486)
  (parent 152)
  (seen 1588 1595)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 487)
  (parent 152)
  (seen 443 451 457 458)
  (unrealized (0 1) (2 0) (6 0))
  (comment "5 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 488)
  (parent 152)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 489)
  (parent 153)
  (seen 405)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 490)
  (parent 153)
  (seen 1607)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 491)
  (parent 153)
  (seen 415 942)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 492)
  (parent 153)
  (seen 942)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 494)
  (parent 154)
  (seen 401)
  (unrealized (0 1) (2 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 495)
  (parent 154)
  (seen 405 1612)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 496)
  (parent 154)
  (seen 406)
  (unrealized (0 1) (6 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 497)
  (parent 154)
  (seen 406 1621)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 499)
  (parent 155)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 500)
  (parent 155)
  (seen 402 1644)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 501)
  (parent 155)
  (seen 403 1524)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 503)
  (parent 157)
  (seen 1529 1634)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 505)
  (parent 157)
  (seen 411 412 413 414 1531)
  (unrealized (0 1) (4 0))
  (comment "5 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 507)
  (parent 158)
  (seen 411)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 508)
  (parent 158)
  (seen 415 1529)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 510)
  (parent 158)
  (seen 411 1661 1662 1664 1665)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 512)
  (parent 159)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 513)
  (parent 159)
  (seen 412 1672)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 515)
  (parent 159)
  (seen 411 1661 1662 1664 1665)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 517)
  (parent 160)
  (seen 1027 1682)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 518)
  (parent 160)
  (seen 520 1687)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 519)
  (parent 160)
  (seen 523 636 1691)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 520)
  (parent 161)
  (seen 1212 1216 1217)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 521)
  (parent 161)
  (seen 1689)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 5 3 resp2 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 522)
  (parent 161)
  (seen 1094 1096 1696)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 523)
  (parent 162)
  (seen 264 658 1228 1232)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 524)
  (parent 162)
  (seen 1693)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 525)
  (parent 163)
  (seen 527 528 1216 1240)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 526)
  (parent 163)
  (seen 1243 1248 1288 1706)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 527)
  (parent 163)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 528)
  (parent 163)
  (seen 1094)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 529)
  (parent 164)
  (seen 264 531 532 1266)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 530)
  (parent 164)
  (seen 1269 1274 1298 1720)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 531)
  (parent 164)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 532)
  (parent 164)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 533)
  (parent 165)
  (seen 1331)
  (unrealized (0 1) (5 0) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 534)
  (parent 166)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (4 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 536)
  (parent 167)
  (unrealized (0 1))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (4 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 537)
  (parent 167)
  (seen 1742)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (4 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 538)
  (parent 167)
  (seen 1747)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (4 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 539)
  (parent 167)
  (seen 1752)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 540)
  (parent 168)
  (seen 543 1217 1357)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 541)
  (parent 168)
  (seen 1365 1374 1757)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 542)
  (parent 168)
  (seen 1216 1217 1409 1760)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 543)
  (parent 168)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 544)
  (parent 168)
  (seen 1096 1759 1763)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 545)
  (parent 169)
  (seen 548 779 1232 1383)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 546)
  (parent 169)
  (seen 779 1389 1395)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 547)
  (parent 169)
  (seen 264 1232 1424 1775)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 548)
  (parent 169)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 549)
  (parent 169)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 550)
  (parent 170)
  (seen 583 1787)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 551)
  (parent 170)
  (seen 542 1792)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 552)
  (parent 170)
  (seen 547 665 1797)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 553)
  (parent 172)
  (seen 133 1022 1802)
  (unrealized (0 1) (6 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 554)
  (parent 172)
  (seen 21 159 480 559 1802)
  (unrealized (0 1) (2 0) (6 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 555)
  (parent 172)
  (seen 1801)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 557)
  (parent 172)
  (seen 133 1573)
  (unrealized (0 1) (6 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 558)
  (parent 172)
  (seen 1573 1822)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 559)
  (parent 172)
  (seen 584 1805)
  (unrealized (0 1) (2 0) (6 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 560)
  (parent 173)
  (seen 443 451 563 1480)
  (unrealized (0 1) (2 0) (6 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 561)
  (parent 173)
  (seen 567)
  (unrealized (0 1) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 562)
  (parent 173)
  (seen 457)
  (unrealized (0 1) (2 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 563)
  (parent 173)
  (seen 573)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 564)
  (parent 174)
  (seen 1469 1470 1471)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 565)
  (parent 174)
  (seen 1444)
  (unrealized (0 1) (5 0) (6 0))
  (comment "6 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 566)
  (parent 174)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 567)
  (parent 175)
  (seen 564)
  (unrealized (0 1) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 569)
  (parent 175)
  (seen 566)
  (unrealized (0 1) (2 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 570)
  (parent 176)
  (seen 1842 1856)
  (unrealized (0 1) (5 0) (6 0))
  (comment "6 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 571)
  (parent 176)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (contracted (nb-0 nc-1)) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 572)
  (parent 177)
  (seen 1859 1860 1861)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 573)
  (parent 177)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 574)
  (parent 177)
  (seen 1859 1860 1861)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 575)
  (parent 177)
  (seen 1498)
  (unrealized (0 1) (2 0) (6 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 576)
  (parent 178)
  (seen 1008 1870)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 577)
  (parent 178)
  (seen 1875)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-2 (4 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 578)
  (parent 179)
  (seen 15 16 309)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 579)
  (parent 179)
  (seen 60 581 582)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 580)
  (parent 179)
  (seen 559 600 1882)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 581)
  (parent 180)
  (seen 47 411 583)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 582)
  (parent 180)
  (seen 53 507 1882)
  (unrealized (0 1) (4 0) (5 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 583)
  (parent 180)
  (seen 160 161 162)
  (unrealized (0 1))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 584)
  (parent 180)
  (seen 1883)
  (unrealized (0 1) (4 0))
  (comment "6 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 585)
  (parent 181)
  (seen 224 589 850)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 586)
  (parent 181)
  (seen 200 590 660 713 1899)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 587)
  (parent 181)
  (seen 200 850 1900)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 588)
  (parent 181)
  (seen 202 615 657 713 1904)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 1 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 589)
  (parent 181)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 590)
  (parent 181)
  (seen 1902)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 591)
  (parent 182)
  (seen 87 259 595 713)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 592)
  (parent 182)
  (seen 224 596 1917)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 593)
  (parent 182)
  (seen 87 1918 1920)
  (unrealized (0 1) (5 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 594)
  (parent 182)
  (seen 224 614 852 1923)
  (unrealized (0 1) (5 0))
  (comment "6 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 595)
  (parent 182)
  (seen 1912 1913 1922 1925)
  (unrealized (0 1) (5 0))
  (comment "7 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 596)
  (parent 182)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 597)
  (parent 183)
  (seen 1053 1935 1936)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 598)
  (parent 183)
  (seen 1001 1941 1942)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 6 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 599)
  (parent 183)
  (seen 993 1188 1947 1948)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 600)
  (parent 184)
  (seen 21 159 559)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 601)
  (parent 184)
  (seen 133 1805 1952)
  (unrealized (0 1) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 602)
  (parent 184)
  (seen 558 1885 1957)
  (unrealized (0 1) (4 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 604)
  (parent 184)
  (seen 133 1027 1818)
  (unrealized (0 1) (6 0))
  (comment "7 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 605)
  (parent 184)
  (seen 558 1027)
  (unrealized (0 1) (4 0))
  (comment "7 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nc-0)) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 606)
  (parent 185)
  (unrealized (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (4 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 607)
  (parent 185)
  (seen 1974 1975 1976)
  (unrealized (0 1))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 608)
  (parent 185)
  (seen 623 624 1883 1972 1977)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 2 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 609)
  (parent 185)
  (seen 559 1963 1972 1979)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 4 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 610)
  (parent 185)
  (seen 563 627 628 1983)
  (unrealized (0 1) (4 0) (6 0))
  (comment "8 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 611)
  (parent 186)
  (seen 1986 1990)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 612)
  (parent 186)
  (seen 613 1991 1992)
  (unrealized (0 1) (6 0))
  (comment "9 in cohort - 5 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 613)
  (parent 187)
  (seen 202 852 1904 1923)
  (unrealized (0 1) (5 0))
  (comment "9 in cohort - 3 not yet seen"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 614)
  (parent 187)
  (seen 1994)
  (unrealized (0 1))
  (comment "3 in cohort - 2 not yet seen"))

(comment "Step limit exceeded--aborting run")

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 615)
  (parent 187)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 616)
  (parent 188)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 617)
  (parent 188)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 618)
  (parent 188)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 619)
  (parent 189)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 620)
  (parent 189)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 621)
  (parent 189)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 622)
  (parent 191)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 623)
  (parent 191)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 624)
  (parent 191)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-2 (5 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 625)
  (parent 191)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 626)
  (parent 192)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 627)
  (parent 192)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 628)
  (parent 192)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 629)
  (parent 192)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 630)
  (parent 193)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 631)
  (parent 194)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 632)
  (parent 194)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 633)
  (parent 194)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 634)
  (parent 195)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 635)
  (parent 195)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 636)
  (parent 198)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 4 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d)))))
  (label 637)
  (parent 198)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 638)
  (parent 199)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 639)
  (parent 199)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 640)
  (parent 199)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 4 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d)))))
  (label 641)
  (parent 199)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 642)
  (parent 200)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (3 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 643)
  (parent 200)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 644)
  (parent 201)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 645)
  (parent 201)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 646)
  (parent 201)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 647)
  (parent 202)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 648)
  (parent 202)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 649)
  (parent 202)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 650)
  (parent 203)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-1 (5 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 651)
  (parent 203)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 652)
  (parent 203)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 653)
  (parent 203)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 654)
  (parent 205)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 655)
  (parent 205)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 656)
  (parent 205)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 657)
  (parent 205)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 658)
  (parent 205)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d)))))
  (label 659)
  (parent 205)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 660)
  (parent 206)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 661)
  (parent 206)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 662)
  (parent 206)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d)))))
  (label 663)
  (parent 206)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (4 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 664)
  (parent 207)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (4 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 665)
  (parent 207)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (4 0)
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c)))))
  (label 666)
  (parent 207)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d)))))
  (label 667)
  (parent 208)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 668)
  (parent 208)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nd nc text) (a b c d name))
  (defstrand init 3 (na nd) (nb nd) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd) (nb nd) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd) (nb nd) (nc nd) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd) (nb nd) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nd) (nb nd) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd)
  (operation nonce-test (contracted (nb nd) (nc-0 nd) (nd-0 nd)) nd
    (0 1) (enc a b c nd nd nd (pubk d)) (enc a b c nd nd nc (pubk d))
    (enc a b d nd nd (pubk c)) (enc a c d nd (pubk b))
    (enc b c d nd nd nd nd (pubk a)))
  (traces
    ((send (enc a c d nd (pubk b)))
      (recv (enc b c d nd nd nd nd (pubk a)))
      (send (enc nd nd nd (pubk b))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nd (pubk c))))
    ((recv (enc a b d nd nd (pubk c)))
      (send (enc a b c nd nd nd (pubk d))))
    ((recv (enc a b c nd nd nd (pubk d)))
      (send (enc b c d nd nd nd nd (pubk a))))
    ((recv (enc a b d nd nd (pubk c)))
      (send (enc a b c nd nd nc (pubk d)))))
  (label 669)
  (parent 209)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nd-0 nd-0 (pubk d))
    (enc a b c nd-0 nd-0 nc-0 (pubk d)) (enc a b d nd-0 nd-0 (pubk c))
    (enc a c d nd-0 (pubk b)) (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-1 (pubk a)))))
  (label 670)
  (parent 209)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nd-0 nd-0 (pubk d))
    (enc a b c nd-0 nd-0 nc-0 (pubk d)) (enc a b d nd-0 nd-0 (pubk c))
    (enc a c d nd-0 (pubk b)) (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nc-0 nd-1 (pubk a)))))
  (label 671)
  (parent 209)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d)))))
  (label 672)
  (parent 210)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 673)
  (parent 210)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 674)
  (parent 211)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-1 (pubk c)))))
  (label 675)
  (parent 211)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 676)
  (parent 211)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 677)
  (parent 211)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 678)
  (parent 211)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nd-0 nc-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nc-1 nc-0)) na (3 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-0 (pubk c)))))
  (label 679)
  (parent 212)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 680)
  (parent 212)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-0 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 681)
  (parent 212)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 682)
  (parent 213)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 683)
  (parent 213)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 684)
  (parent 213)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 685)
  (parent 213)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 686)
  (parent 214)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 687)
  (parent 214)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 688)
  (parent 215)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 689)
  (parent 215)
  (unrealized (0 1) (3 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 690)
  (parent 215)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 691)
  (parent 215)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 692)
  (parent 215)
  (unrealized (0 1) (3 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 693)
  (parent 215)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 694)
  (parent 216)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 695)
  (parent 217)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 696)
  (parent 217)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 697)
  (parent 217)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d)))))
  (label 698)
  (parent 218)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 699)
  (parent 218)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 700)
  (parent 218)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 701)
  (parent 219)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 702)
  (parent 219)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-1 (pubk a)))))
  (label 703)
  (parent 219)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 704)
  (parent 221)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 705)
  (parent 221)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 706)
  (parent 221)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 707)
  (parent 221)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 708)
  (parent 221)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 709)
  (parent 221)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 710)
  (parent 223)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 711)
  (parent 223)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 712)
  (parent 223)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 713)
  (parent 223)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-1 (pubk d)))))
  (label 714)
  (parent 223)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 715)
  (parent 224)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 716)
  (parent 224)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 717)
  (parent 224)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nd nb nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d)))))
  (label 718)
  (parent 225)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 719)
  (parent 225)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 720)
  (parent 225)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 721)
  (parent 226)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 722)
  (parent 226)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (3 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 723)
  (parent 226)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 724)
  (parent 227)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 725)
  (parent 227)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 726)
  (parent 228)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 727)
  (parent 228)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 728)
  (parent 228)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 729)
  (parent 228)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 730)
  (parent 229)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 731)
  (parent 229)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 732)
  (parent 229)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 733)
  (parent 229)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 734)
  (parent 229)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 735)
  (parent 230)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp1 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 736)
  (parent 230)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (5 0)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 737)
  (parent 230)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 738)
  (parent 231)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 739)
  (parent 231)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 740)
  (parent 231)
  (unrealized (0 1) (3 0) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 741)
  (parent 231)
  (unrealized (0 1) (3 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 742)
  (parent 231)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 743)
  (parent 231)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nd nb nc text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b d na nb (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d)))))
  (label 744)
  (parent 232)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d)))))
  (label 745)
  (parent 233)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 746)
  (parent 233)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (3 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 747)
  (parent 233)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a b c na nb nc (pubk d)))
      (send (enc b c d na nb nc nd (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d)))))
  (label 748)
  (parent 234)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-1 (pubk a)))))
  (label 749)
  (parent 234)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d))
    (enc a b c nc nb-0 nc-0 (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b d nc nb-0 (pubk c)))
      (send (enc a b c nc nb-0 nc-0 (pubk d)))))
  (label 750)
  (parent 235)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 751)
  (parent 235)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 752)
  (parent 235)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 753)
  (parent 236)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a)))))
  (label 754)
  (parent 236)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 755)
  (parent 236)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 756)
  (parent 236)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 757)
  (parent 236)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 758)
  (parent 236)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 759)
  (parent 236)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 760)
  (parent 237)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 761)
  (parent 237)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 762)
  (parent 237)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 763)
  (parent 237)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 764)
  (parent 238)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 765)
  (parent 238)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (3 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 766)
  (parent 238)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c)))))
  (label 767)
  (parent 238)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 768)
  (parent 238)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (3 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 769)
  (parent 238)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 770)
  (parent 238)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp1 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-2 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 771)
  (parent 240)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-2 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c))))
    ((recv (enc a b d nc-0 nb-2 (pubk c)))
      (send (enc a b c nc-0 nb-2 nc-2 (pubk d)))))
  (label 772)
  (parent 240)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 773)
  (parent 241)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 774)
  (parent 241)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 775)
  (parent 241)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 776)
  (parent 241)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (3 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 777)
  (parent 241)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 778)
  (parent 242)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 779)
  (parent 243)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a b c nb-1 nb-1 nc-0 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 780)
  (parent 243)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (5 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 781)
  (parent 243)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 782)
  (parent 244)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 783)
  (parent 244)
  (unrealized (0 1) (3 0) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 784)
  (parent 244)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 785)
  (parent 244)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nd nb nc text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b d nb nb (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d)))))
  (label 786)
  (parent 245)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 787)
  (parent 245)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 788)
  (parent 246)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 789)
  (parent 246)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 790)
  (parent 246)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0) (nc-1 nc-0)) nb-0 (3 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 791)
  (parent 247)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (3 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 792)
  (parent 247)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 793)
  (parent 247)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (3 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 794)
  (parent 247)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (3 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 795)
  (parent 247)
  (unrealized (0 1) (3 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b c nb nb nc-0 (pubk d))
    (enc a b d nb nb (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc-0 (pubk d)))))
  (label 796)
  (parent 248)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 797)
  (parent 248)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 798)
  (parent 248)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d)))))
  (label 799)
  (parent 249)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 800)
  (parent 249)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 801)
  (parent 249)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 802)
  (parent 250)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 803)
  (parent 250)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 804)
  (parent 250)
  (unrealized (0 1) (3 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 805)
  (parent 250)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (6 0)) ((6 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 5 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-1 (pubk a)))))
  (label 806)
  (parent 250)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d))
    (enc a b c nc nb-0 nc-0 (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b d nc nb-0 (pubk c)))
      (send (enc a b c nc nb-0 nc-0 (pubk d)))))
  (label 807)
  (parent 251)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 808)
  (parent 251)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 809)
  (parent 251)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nb-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nb-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nb-0) (nd-0 nd))
    nb-0 (0 1) (enc a b c nb-0 nb nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb nb-0 nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nb-0 nd (pubk a)))
      (send (enc nb nb-0 nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb nb-0 (pubk d)))
      (send (enc b c d nb-0 nb nb-0 nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc (pubk d)))))
  (label 810)
  (parent 252)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nb-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 811)
  (parent 252)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nb-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-1 (pubk a)))))
  (label 812)
  (parent 252)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 813)
  (parent 253)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 814)
  (parent 253)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 815)
  (parent 253)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 816)
  (parent 253)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 817)
  (parent 253)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 818)
  (parent 254)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 819)
  (parent 254)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 820)
  (parent 254)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-1 (pubk a)))))
  (label 821)
  (parent 254)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (6 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-1 (pubk a)))))
  (label 822)
  (parent 254)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 823)
  (parent 254)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 824)
  (parent 255)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp1 2) nc-1 (5 0)
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d)))))
  (label 825)
  (parent 255)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 826)
  (parent 255)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d))
    (enc a b c nc nb-0 nc-0 (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b d nc nb-0 (pubk c)))
      (send (enc a b c nc nb-0 nc-0 (pubk d)))))
  (label 827)
  (parent 256)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 828)
  (parent 256)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 829)
  (parent 256)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nc-0 (5 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 830)
  (parent 257)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (5 0)
    (enc a b d nc-1 nb-2 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 831)
  (parent 257)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp1 2) nc-1 (5 0)
    (enc a b d nc-1 nb-2 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 832)
  (parent 257)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-2 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c))))
    ((recv (enc a b d nc-0 nb-2 (pubk c)))
      (send (enc a b c nc-0 nb-2 nc-2 (pubk d)))))
  (label 833)
  (parent 257)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 7 3 resp3 2) nc-1 (5 0)
    (enc a b d nc-1 nb-2 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-2 (pubk c)))))
  (label 834)
  (parent 257)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 835)
  (parent 258)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nb-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 836)
  (parent 258)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 837)
  (parent 258)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nb-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb) (nc nb-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nb-0) (nd-0 nd))
    nb-0 (0 1) (enc a b c nb-0 nb nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc (pubk d)) (enc a b d nb-0 nb (pubk c))
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb nb-0 nd (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nb-0 nd (pubk a)))
      (send (enc nb nb-0 nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb (pubk c))))
    ((recv (enc a b d nb-0 nb (pubk c)))
      (send (enc a b c nb-0 nb nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb nb-0 (pubk d)))
      (send (enc b c d nb-0 nb nb-0 nd (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc (pubk d)))))
  (label 838)
  (parent 259)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nb-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d)))))
  (label 839)
  (parent 259)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nb-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-0 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b))
    (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-1 (pubk a)))))
  (label 840)
  (parent 259)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp1 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 841)
  (parent 260)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp1 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d)))))
  (label 842)
  (parent 260)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 843)
  (parent 260)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (5 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 844)
  (parent 260)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nb-0 nc text) (a b c d name))
  (defstrand init 3 (na nd) (nb nb) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd) (nb nb) (nc nd) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd) (nb nb) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd) (nb nb-0) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nd) (nd-0 nd)) nd
    (0 1) (enc a b c nd nb nd (pubk d)) (enc a b c nd nb-0 nc (pubk d))
    (enc a b d nd nb (pubk c)) (enc a b d nd nb-0 (pubk c))
    (enc a c d nd (pubk b)) (enc b c d nd nb nd nd (pubk a)))
  (traces
    ((send (enc a c d nd (pubk b)))
      (recv (enc b c d nd nb nd nd (pubk a)))
      (send (enc nb nd nd (pubk b))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nb (pubk c))))
    ((recv (enc a b d nd nb (pubk c)))
      (send (enc a b c nd nb nd (pubk d))))
    ((recv (enc a b c nd nb nd (pubk d)))
      (send (enc b c d nd nb nd nd (pubk a))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nb-0 (pubk c))))
    ((recv (enc a b d nd nb-0 (pubk c)))
      (send (enc a b c nd nb-0 nc (pubk d)))))
  (label 845)
  (parent 261)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp2 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nd-0 (pubk d))
    (enc a b c nd-0 nb-1 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nb-1 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-1 (pubk d)))))
  (label 846)
  (parent 261)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nd-0 (pubk d))
    (enc a b c nd-0 nb-1 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nb-1 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-1 (pubk a)))))
  (label 847)
  (parent 261)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nd-0 (pubk d))
    (enc a b c nd-0 nb-1 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nb-1 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-1 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-1 nc-0 nd-1 (pubk a)))))
  (label 848)
  (parent 261)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 849)
  (parent 263)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 850)
  (parent 263)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 851)
  (parent 263)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 852)
  (parent 263)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-1 (pubk d)))))
  (label 853)
  (parent 263)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b c nb nb nc-0 (pubk d))
    (enc a b d nb nb (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a b c nb nb nc (pubk d)))
      (send (enc b c d nb nb nc nd (pubk a))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc-0 (pubk d)))))
  (label 854)
  (parent 264)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 855)
  (parent 264)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 856)
  (parent 264)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 857)
  (parent 265)
  (unrealized (0 1) (3 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 858)
  (parent 265)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 859)
  (parent 265)
  (unrealized (0 1) (3 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (6 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 860)
  (parent 265)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nd nc-0 nb text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c)))))
  (label 861)
  (parent 266)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 862)
  (parent 266)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 863)
  (parent 266)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nd nc-0 nb text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c)))))
  (label 864)
  (parent 268)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 865)
  (parent 268)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 866)
  (parent 268)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nd-1 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 867)
  (parent 269)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((2 1) (5 0))
    ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 868)
  (parent 269)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 869)
  (parent 269)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (5 0))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nc-0 nd-1 (pubk a)))))
  (label 870)
  (parent 269)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 871)
  (parent 271)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 872)
  (parent 271)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 873)
  (parent 271)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 874)
  (parent 271)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 875)
  (parent 271)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 876)
  (parent 271)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nd nc-0 nb text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c)))))
  (label 877)
  (parent 272)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 nb-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 878)
  (parent 272)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nc nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nb) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nb) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nb) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nb (pubk d)) (enc a b c nb nb nc (pubk d))
    (enc a b d nb nb (pubk c)) (enc a b d nb nb-0 (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nb nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nb nd (pubk a)))
      (send (enc nb nb nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nb (pubk d))))
    ((recv (enc a b c nb nb nb (pubk d)))
      (send (enc b c d nb nb nb nd (pubk a))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 879)
  (parent 273)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 880)
  (parent 273)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 881)
  (parent 273)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 882)
  (parent 274)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 5 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 883)
  (parent 274)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (7 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 884)
  (parent 274)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (6 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 885)
  (parent 274)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-0 nb-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 886)
  (parent 274)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 887)
  (parent 274)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 7 3 resp3 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-1 (pubk d)))))
  (label 888)
  (parent 274)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp2 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nd-0 (pubk d))
    (enc a b c nd-0 nb-0 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nb-1 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-1 (pubk d)))))
  (label 889)
  (parent 275)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nc nb-0 text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nb) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nb) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb) (nc-0 nb) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nb (pubk d)) (enc a b c nb nb nc (pubk d))
    (enc a b d nb nb (pubk c)) (enc a b d nb nb-0 (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb nb nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nb nd (pubk a)))
      (send (enc nb nb nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nb (pubk d))))
    ((recv (enc a b c nb nb nb (pubk d)))
      (send (enc b c d nb nb nb nd (pubk a))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b)))
      (send (enc a b d nb nb-0 (pubk c)))))
  (label 890)
  (parent 277)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 891)
  (parent 277)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 892)
  (parent 277)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 893)
  (parent 278)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 894)
  (parent 278)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 895)
  (parent 278)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nc-0 nd-1 (pubk a)))))
  (label 896)
  (parent 278)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 897)
  (parent 280)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 898)
  (parent 280)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 899)
  (parent 280)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-0 (5 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 900)
  (parent 280)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 901)
  (parent 280)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nc-1 (5 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 902)
  (parent 280)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd text) (a b c d name))
  (defstrand init 3 (na nd) (nb nb) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd) (nb nb) (nc nd) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd) (nb nb) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd)
  (operation generalization deleted (4 0))
  (traces
    ((send (enc a c d nd (pubk b)))
      (recv (enc b c d nd nb nd nd (pubk a)))
      (send (enc nb nd nd (pubk b))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nb (pubk c))))
    ((recv (enc a b d nd nb (pubk c)))
      (send (enc a b c nd nb nd (pubk d))))
    ((recv (enc a b c nd nb nd (pubk d)))
      (send (enc b c d nd nb nd nd (pubk a)))))
  (label 903)
  (parent 281)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 1 resp1 2) nd-0 (5 0)
    (enc a c d nd-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-1 (pubk a)))))
  (label 904)
  (parent 282)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp1 2) nd-0 (5 0)
    (enc a c d nd-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-1 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c)))))
  (label 905)
  (parent 282)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nb-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nd-1 (5 0)
    (enc a c d nd-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nb-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 906)
  (parent 282)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 2 resp2 2) nd-0 (5 0)
    (enc a c d nd-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-1 (pubk a)))))
  (label 907)
  (parent 282)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nc-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nd-1 (5 0)
    (enc a c d nd-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nc-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 908)
  (parent 282)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nd-0 (5 0)
    (enc a c d nd-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-1 (pubk a)))))
  (label 909)
  (parent 282)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 1 resp1 2) nd-0 (5 0)
    (enc a c d nd-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 910)
  (parent 283)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (6 0)) ((1 1) (2 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp1 2) nd-0 (5 0)
    (enc a c d nd-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c)))))
  (label 911)
  (parent 283)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nb-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nd-1 (5 0)
    (enc a c d nd-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nb-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 912)
  (parent 283)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 2 resp2 2) nd-0 (5 0)
    (enc a c d nd-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 913)
  (parent 283)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 3 resp3 2) nd-0 (5 0)
    (enc a c d nd-0 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 914)
  (parent 283)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 915)
  (parent 284)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 916)
  (parent 284)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 917)
  (parent 284)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 918)
  (parent 284)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 919)
  (parent 284)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 920)
  (parent 284)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 921)
  (parent 285)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 922)
  (parent 285)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 923)
  (parent 285)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 924)
  (parent 285)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 925)
  (parent 285)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 926)
  (parent 286)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (6 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 927)
  (parent 286)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 928)
  (parent 288)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 929)
  (parent 288)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 930)
  (parent 288)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 931)
  (parent 288)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 932)
  (parent 289)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 933)
  (parent 289)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 934)
  (parent 289)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 935)
  (parent 289)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 936)
  (parent 290)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 937)
  (parent 290)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 938)
  (parent 290)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 939)
  (parent 291)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 940)
  (parent 292)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 941)
  (parent 292)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 942)
  (parent 292)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-3 (pubk d)))))
  (label 943)
  (parent 292)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 944)
  (parent 292)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (1 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 945)
  (parent 293)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 946)
  (parent 294)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-2 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-2 nd-0 (pubk a)))))
  (label 947)
  (parent 294)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 948)
  (parent 294)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 949)
  (parent 295)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 950)
  (parent 295)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 951)
  (parent 295)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 952)
  (parent 295)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 953)
  (parent 295)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 954)
  (parent 296)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 955)
  (parent 297)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 956)
  (parent 297)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 957)
  (parent 297)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 958)
  (parent 297)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 959)
  (parent 298)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 960)
  (parent 299)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 961)
  (parent 299)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 962)
  (parent 299)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-0 nb-0) (nc-1 nb-0)) nb-0 (5 0)
    (enc a b c nb-0 nb-0 nb-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 963)
  (parent 300)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (4 0)) ((2 1) (1 0)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 964)
  (parent 300)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 965)
  (parent 300)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 966)
  (parent 300)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 967)
  (parent 300)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 968)
  (parent 301)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 969)
  (parent 301)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 970)
  (parent 301)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 971)
  (parent 301)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 972)
  (parent 302)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 973)
  (parent 302)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 974)
  (parent 303)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 975)
  (parent 303)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 976)
  (parent 303)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 977)
  (parent 303)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 978)
  (parent 304)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 979)
  (parent 304)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 980)
  (parent 305)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 981)
  (parent 305)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 982)
  (parent 305)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 983)
  (parent 305)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 984)
  (parent 305)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 985)
  (parent 305)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb nc-0 (pubk d)))
      (send (enc b c d nc-0 nb nc-0 nd (pubk a)))))
  (label 986)
  (parent 306)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 987)
  (parent 306)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-1 (pubk a)))))
  (label 988)
  (parent 306)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 989)
  (parent 307)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 990)
  (parent 307)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 991)
  (parent 307)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 992)
  (parent 307)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 993)
  (parent 307)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 994)
  (parent 307)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 995)
  (parent 307)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 996)
  (parent 307)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 997)
  (parent 308)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 998)
  (parent 308)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 999)
  (parent 308)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1000)
  (parent 308)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1001)
  (parent 308)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1002)
  (parent 308)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1003)
  (parent 308)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1004)
  (parent 308)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (1 0))
    ((2 1) (3 0)) ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1005)
  (parent 309)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1006)
  (parent 309)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (1 0)) ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1007)
  (parent 309)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1008)
  (parent 309)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1009)
  (parent 309)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1010)
  (parent 310)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1011)
  (parent 310)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 1012)
  (parent 310)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1013)
  (parent 310)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1014)
  (parent 310)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1015)
  (parent 311)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1016)
  (parent 311)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1017)
  (parent 311)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1018)
  (parent 311)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-2 (pubk d)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1019)
  (parent 312)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1020)
  (parent 312)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1021)
  (parent 312)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1022)
  (parent 312)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (4 0)
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1023)
  (parent 312)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1024)
  (parent 312)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1025)
  (parent 314)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1026)
  (parent 314)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (0 1)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1027)
  (parent 315)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1028)
  (parent 315)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1029)
  (parent 315)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1030)
  (parent 315)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1031)
  (parent 315)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-2 nd-0 (pubk a)))))
  (label 1032)
  (parent 315)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nb-0)) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1033)
  (parent 316)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 5 resp1 2) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1034)
  (parent 316)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nc-2 (6 0)
    (enc a b c nc-2 nc-2 nc-2 (pubk d)) (enc a b d nc-2 nc-2 (pubk c))
    (enc a b d nc-2 nb-0 (pubk c)) (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1035)
  (parent 316)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-3 (pubk d)))))
  (label 1036)
  (parent 316)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-3 (pubk d)))))
  (label 1037)
  (parent 316)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1038)
  (parent 316)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1039)
  (parent 317)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1040)
  (parent 317)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 5 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1041)
  (parent 317)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1042)
  (parent 317)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1043)
  (parent 317)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1044)
  (parent 317)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1045)
  (parent 317)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp2 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1046)
  (parent 317)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1047)
  (parent 318)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-2 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-2 nd-0 (pubk a)))))
  (label 1048)
  (parent 318)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1049)
  (parent 318)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1050)
  (parent 319)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1051)
  (parent 319)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-3 (pubk d)))))
  (label 1052)
  (parent 319)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nc-2 nb-1)) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c)))))
  (label 1053)
  (parent 320)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1054)
  (parent 320)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-2 (5 0)
    (enc a b d nc-2 nb-1 (pubk c)) (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1055)
  (parent 320)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nc-3 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-3 (pubk d)))))
  (label 1056)
  (parent 320)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp2 2) nc-2 (5 0)
    (enc a b d nc-2 nb-1 (pubk c)) (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1057)
  (parent 320)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (1 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nb-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1058)
  (parent 321)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1059)
  (parent 322)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-2 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-2 nd-0 (pubk a)))))
  (label 1060)
  (parent 322)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1061)
  (parent 322)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1062)
  (parent 323)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1063)
  (parent 323)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1064)
  (parent 323)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1065)
  (parent 323)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1066)
  (parent 324)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1067)
  (parent 324)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 1068)
  (parent 324)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1069)
  (parent 324)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1070)
  (parent 325)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-1 (5 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1071)
  (parent 326)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1072)
  (parent 326)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp2 2) nc-1 (5 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1073)
  (parent 326)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (1 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1074)
  (parent 327)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (1 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1075)
  (parent 327)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1076)
  (parent 327)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 1077)
  (parent 328)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1078)
  (parent 328)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1079)
  (parent 328)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nc nc (pubk d))
    (enc a b d nc nb (pubk c)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 1080)
  (parent 329)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 1081)
  (parent 329)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 1082)
  (parent 329)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1083)
  (parent 329)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1084)
  (parent 330)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1085)
  (parent 330)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1086)
  (parent 330)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1087)
  (parent 330)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1088)
  (parent 331)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1089)
  (parent 331)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-1 (5 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1090)
  (parent 332)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1091)
  (parent 332)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 4 resp2 2) nc-1 (5 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1092)
  (parent 332)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (1 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1093)
  (parent 333)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1094)
  (parent 333)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (1 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1095)
  (parent 333)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (3 0)) ((2 1) (4 0))
    ((3 1) (0 1)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nc-0) (nc-1 nc-0)) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1096)
  (parent 334)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1097)
  (parent 334)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1098)
  (parent 334)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 1099)
  (parent 334)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-0 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nc nc (pubk d))
    (enc a b d nc nb (pubk c)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 1100)
  (parent 335)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 1101)
  (parent 335)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 1102)
  (parent 335)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 1103)
  (parent 335)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1104)
  (parent 336)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1105)
  (parent 336)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1106)
  (parent 337)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1107)
  (parent 337)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1108)
  (parent 338)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1109)
  (parent 338)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1110)
  (parent 338)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 1111)
  (parent 338)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 1112)
  (parent 339)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 1113)
  (parent 339)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 1114)
  (parent 339)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1115)
  (parent 339)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1116)
  (parent 341)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1117)
  (parent 341)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1118)
  (parent 341)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 6 4 resp3 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-1 (pubk d)))))
  (label 1119)
  (parent 341)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nb-1 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1120)
  (parent 342)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 1121)
  (parent 342)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1122)
  (parent 342)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1123)
  (parent 343)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1124)
  (parent 343)
  (unrealized (0 1) (4 0) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1125)
  (parent 344)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1126)
  (parent 344)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1127)
  (parent 344)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1128)
  (parent 344)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1129)
  (parent 344)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 1130)
  (parent 345)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d)))))
  (label 1131)
  (parent 346)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1132)
  (parent 346)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 1133)
  (parent 346)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1134)
  (parent 347)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1135)
  (parent 347)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1136)
  (parent 347)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1137)
  (parent 347)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1138)
  (parent 347)
  (unrealized (0 1) (4 0) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1139)
  (parent 348)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-0 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 1140)
  (parent 348)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1141)
  (parent 348)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1142)
  (parent 349)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1143)
  (parent 349)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1144)
  (parent 349)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1145)
  (parent 349)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d)))))
  (label 1146)
  (parent 351)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1147)
  (parent 351)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1148)
  (parent 351)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 1149)
  (parent 353)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 1150)
  (parent 353)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 1151)
  (parent 353)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1152)
  (parent 353)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 1153)
  (parent 353)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1154)
  (parent 353)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 1155)
  (parent 354)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1156)
  (parent 354)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1157)
  (parent 354)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-1) (nc-1 nc-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-0 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d)))))
  (label 1158)
  (parent 355)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-1 (pubk d)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d)))))
  (label 1159)
  (parent 355)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1160)
  (parent 355)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-1 (pubk d)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1161)
  (parent 355)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (4 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1162)
  (parent 355)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nd-1 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-2) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-1 (pubk d)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-2 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-2 nc-1 nd-1 (pubk a)))))
  (label 1163)
  (parent 355)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1164)
  (parent 357)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 nb-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d)))))
  (label 1165)
  (parent 357)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1166)
  (parent 358)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1167)
  (parent 358)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 1168)
  (parent 358)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1169)
  (parent 358)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 5 resp1 2) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1170)
  (parent 359)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nc-1 (6 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d)))))
  (label 1171)
  (parent 359)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1172)
  (parent 359)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1173)
  (parent 359)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (6 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 1174)
  (parent 359)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1175)
  (parent 360)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1176)
  (parent 360)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 5 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1177)
  (parent 360)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1178)
  (parent 360)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d)))))
  (label 1179)
  (parent 360)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nb-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d)))))
  (label 1180)
  (parent 360)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (6 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d)))))
  (label 1181)
  (parent 360)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nb-1 nc-0 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 7 4 resp3 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-0 (pubk d)))))
  (label 1182)
  (parent 360)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d)))))
  (label 1183)
  (parent 361)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1184)
  (parent 361)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 1185)
  (parent 361)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d)))))
  (label 1186)
  (parent 362)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1187)
  (parent 362)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nc-1 nb-1)) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nb-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c)))))
  (label 1188)
  (parent 363)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1189)
  (parent 363)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-1 (5 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1190)
  (parent 363)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1191)
  (parent 363)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 7 4 resp3 2) nc-1 (5 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c)))))
  (label 1192)
  (parent 363)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nd nc-0 text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d)))))
  (label 1193)
  (parent 365)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1194)
  (parent 365)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((2 1) (5 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 1195)
  (parent 365)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nc text) (a b c d name))
  (defstrand init 3 (na nd) (nb nd) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd) (nb nd) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd) (nb nd) (nc nd) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nd) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd) (nb nd) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nd) (nb nd) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd)
  (operation nonce-test (contracted (nb-0 nd) (nc-0 nd) (nd-0 nd)) nd
    (0 1) (enc a b c nd nd nd (pubk d)) (enc a b c nd nd nc (pubk d))
    (enc a b d nd nb (pubk c)) (enc a b d nd nd (pubk c))
    (enc a c d nd (pubk b)) (enc b c d nd nd nd nd (pubk a)))
  (traces
    ((send (enc a c d nd (pubk b)))
      (recv (enc b c d nd nd nd nd (pubk a)))
      (send (enc nd nd nd (pubk b))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nd (pubk c))))
    ((recv (enc a b d nd nd (pubk c)))
      (send (enc a b c nd nd nd (pubk d))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nb (pubk c))))
    ((recv (enc a b c nd nd nd (pubk d)))
      (send (enc b c d nd nd nd nd (pubk a))))
    ((recv (enc a b d nd nd (pubk c)))
      (send (enc a b c nd nd nc (pubk d)))))
  (label 1196)
  (parent 366)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp2 2) nd-0 (0 1)
    (enc a b c nd-0 nd-0 nd-0 (pubk d))
    (enc a b c nd-0 nd-0 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nd-0 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-1 (pubk d)))))
  (label 1197)
  (parent 366)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nd-0 nd-0 (pubk d))
    (enc a b c nd-0 nd-0 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nd-0 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-1 (pubk a)))))
  (label 1198)
  (parent 366)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nd-0 nd-0 (pubk d))
    (enc a b c nd-0 nd-0 nc-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nd-0 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nc-0 nd-1 (pubk a)))))
  (label 1199)
  (parent 366)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (5 0)
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1200)
  (parent 367)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-1 (5 0)
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d)))))
  (label 1201)
  (parent 367)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1202)
  (parent 367)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1203)
  (parent 368)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 7 4 resp3 2) nc-1 (5 0)
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c)))))
  (label 1204)
  (parent 368)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nd nc text) (a b c d name))
  (defstrand init 3 (na nd) (nb nd) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd) (nb nd) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd) (nb nd) (nc nd) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nd) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd) (nb nd) (nc nd) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nd) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd)
  (operation nonce-test (contracted (nb-0 nd) (nc-0 nd) (nd-0 nd)) nd
    (0 1) (enc a b c nd nb nc (pubk d)) (enc a b c nd nd nd (pubk d))
    (enc a b d nd nb (pubk c)) (enc a b d nd nd (pubk c))
    (enc a c d nd (pubk b)) (enc b c d nd nd nd nd (pubk a)))
  (traces
    ((send (enc a c d nd (pubk b)))
      (recv (enc b c d nd nd nd nd (pubk a)))
      (send (enc nd nd nd (pubk b))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nd (pubk c))))
    ((recv (enc a b d nd nd (pubk c)))
      (send (enc a b c nd nd nd (pubk d))))
    ((recv (enc a c d nd (pubk b))) (send (enc a b d nd nb (pubk c))))
    ((recv (enc a b c nd nd nd (pubk d)))
      (send (enc b c d nd nd nd nd (pubk a))))
    ((recv (enc a b d nd nb (pubk c)))
      (send (enc a b c nd nb nc (pubk d)))))
  (label 1205)
  (parent 369)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp2 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nc-0 (pubk d))
    (enc a b c nd-0 nd-0 nd-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nd-0 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-1 (pubk d)))))
  (label 1206)
  (parent 369)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nc-0 (pubk d))
    (enc a b c nd-0 nd-0 nd-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nd-0 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nd-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1207)
  (parent 369)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 nc-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (added-strand resp3 2) nd-0 (0 1)
    (enc a b c nd-0 nb-0 nc-0 (pubk d))
    (enc a b c nd-0 nd-0 nd-0 (pubk d)) (enc a b d nd-0 nb-0 (pubk c))
    (enc a b d nd-0 nd-0 (pubk c)) (enc a c d nd-0 (pubk b))
    (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-1 (pubk a)))))
  (label 1208)
  (parent 369)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 2 resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1209)
  (parent 370)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 3 resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1210)
  (parent 370)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1211)
  (parent 370)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1212)
  (parent 371)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1213)
  (parent 371)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1214)
  (parent 371)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1215)
  (parent 371)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 2 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1216)
  (parent 372)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1217)
  (parent 372)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1218)
  (parent 372)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 1219)
  (parent 373)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1220)
  (parent 373)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nc-1 nc-0)) nc-0 (4 0)
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1221)
  (parent 374)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 5 2 resp2 2) nc-1 (4 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1222)
  (parent 374)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (4 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1223)
  (parent 374)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nc-1 (4 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1224)
  (parent 374)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (4 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1225)
  (parent 374)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 5 3 resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a)))))
  (label 1226)
  (parent 375)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1227)
  (parent 375)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1228)
  (parent 376)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1229)
  (parent 376)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1230)
  (parent 376)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) na (4 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1231)
  (parent 376)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (0 1)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 5 3 resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1232)
  (parent 377)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1233)
  (parent 377)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nb nc-0 (pubk d)))
      (send (enc b c d nc nb nc-0 nd (pubk a)))))
  (label 1234)
  (parent 378)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1235)
  (parent 378)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1236)
  (parent 379)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1237)
  (parent 379)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1238)
  (parent 379)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1239)
  (parent 379)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1240)
  (parent 380)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1241)
  (parent 380)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1242)
  (parent 380)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1243)
  (parent 381)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1244)
  (parent 381)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1245)
  (parent 381)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1246)
  (parent 381)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c)))))
  (label 1247)
  (parent 381)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1248)
  (parent 382)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1249)
  (parent 382)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1250)
  (parent 382)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1251)
  (parent 383)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1252)
  (parent 383)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1253)
  (parent 383)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d))
    (enc a b c nc nb-0 nc-0 (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b d nc nb-0 (pubk c)))
      (send (enc a b c nc nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 1254)
  (parent 384)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1255)
  (parent 384)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1256)
  (parent 384)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1257)
  (parent 385)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1258)
  (parent 385)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1259)
  (parent 385)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1260)
  (parent 385)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (5 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1261)
  (parent 385)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1262)
  (parent 386)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1263)
  (parent 386)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1264)
  (parent 386)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1265)
  (parent 386)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1266)
  (parent 387)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1267)
  (parent 387)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1268)
  (parent 387)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1269)
  (parent 388)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1270)
  (parent 388)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1271)
  (parent 388)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1272)
  (parent 388)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c)))))
  (label 1273)
  (parent 388)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1274)
  (parent 389)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1275)
  (parent 389)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1276)
  (parent 389)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1277)
  (parent 390)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1278)
  (parent 390)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1279)
  (parent 390)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0) (nd-0 nd))
    nc (0 1) (enc a b c nc nb nc (pubk d))
    (enc a b c nc nb-0 nc-0 (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb-0 nc-0 nd (pubk a)))
      (send (enc nb-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b d nc nb-0 (pubk c)))
      (send (enc a b c nc nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc nb-0 nc-0 (pubk d)))
      (send (enc b c d nc nb-0 nc-0 nd (pubk a)))))
  (label 1280)
  (parent 391)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1281)
  (parent 391)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1282)
  (parent 391)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc (pubk d))
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))
      (send (enc nb-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))))
  (label 1283)
  (parent 392)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1284)
  (parent 392)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1285)
  (parent 392)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1286)
  (parent 393)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1287)
  (parent 393)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1288)
  (parent 394)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1289)
  (parent 394)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1290)
  (parent 394)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1291)
  (parent 394)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1292)
  (parent 395)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1293)
  (parent 395)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 1294)
  (parent 396)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1295)
  (parent 396)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 1296)
  (parent 396)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1297)
  (parent 397)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1298)
  (parent 398)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1299)
  (parent 398)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1300)
  (parent 398)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1301)
  (parent 398)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1302)
  (parent 399)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1303)
  (parent 399)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc-0) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc-0 nd (pubk a)))
      (send (enc nc nc-0 nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc-0 (pubk d)))
      (send (enc b c d nc nc nc-0 nd (pubk a)))))
  (label 1304)
  (parent 400)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1305)
  (parent 400)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1306)
  (parent 400)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1307)
  (parent 401)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a)))))
  (label 1308)
  (parent 401)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b c na nb-0 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-0 (pubk a)))))
  (label 1309)
  (parent 401)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1310)
  (parent 402)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-0 (pubk a)))))
  (label 1311)
  (parent 402)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1312)
  (parent 402)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-2 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1313)
  (parent 403)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-2 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1314)
  (parent 403)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-2 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1315)
  (parent 403)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-0 (pubk a)))))
  (label 1316)
  (parent 404)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-0 (pubk a)))))
  (label 1317)
  (parent 404)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1318)
  (parent 404)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1319)
  (parent 404)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1320)
  (parent 404)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-2) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-2) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-2)
  (operation nonce-test (displaced 7 5 resp2 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-2 (pubk b)))
      (recv (enc b c d nc-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-2 (pubk b)))
      (send (enc a b d nc-2 nb-0 (pubk c))))
    ((recv (enc a b d nc-2 nb-0 (pubk c)))
      (send (enc a b c nc-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-2 (pubk b)))
      (send (enc a b d nc-2 nb-1 (pubk c))))
    ((recv (enc a b d nc-2 nb-1 (pubk c)))
      (send (enc a b c nc-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-2 nb-0 (pubk c)))
      (send (enc a b c nc-2 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-2 nb-0 nc-2 (pubk d)))
      (send (enc b c d nc-2 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1321)
  (parent 404)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1322)
  (parent 405)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-0 nc-2 (pubk d)))
      (send (enc b c d na nb-0 nc-2 nd-0 (pubk a)))))
  (label 1323)
  (parent 405)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-2 (pubk d))
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1324)
  (parent 405)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1325)
  (parent 406)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1326)
  (parent 406)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1327)
  (parent 406)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1328)
  (parent 407)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1329)
  (parent 407)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1330)
  (parent 407)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1331)
  (parent 407)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d)))))
  (label 1332)
  (parent 407)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1333)
  (parent 408)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1334)
  (parent 408)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-3 (pubk d)))))
  (label 1335)
  (parent 408)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1336)
  (parent 408)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1337)
  (parent 409)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d)))))
  (label 1338)
  (parent 409)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-3 (pubk d)))))
  (label 1339)
  (parent 409)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1340)
  (parent 412)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1341)
  (parent 412)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d))
    (enc a b c nb-0 nb-1 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1342)
  (parent 412)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1343)
  (parent 413)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-3 (pubk d)))))
  (label 1344)
  (parent 413)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1345)
  (parent 413)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1346)
  (parent 414)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1347)
  (parent 414)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1348)
  (parent 414)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-2) (nb nc-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-2) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-2) (nb nc-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (2 0)) ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-2)
  (operation nonce-test (displaced 7 5 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-2 (pubk b)))
      (recv (enc b c d nc-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-2 (pubk b)))
      (send (enc a b d nc-2 nc-2 (pubk c))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-0 (pubk d))))
    ((recv (enc a c d nc-2 (pubk b)))
      (send (enc a b d nc-2 nb-0 (pubk c))))
    ((recv (enc a b d nc-2 nb-0 (pubk c)))
      (send (enc a b c nc-2 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-2 (pubk d))))
    ((recv (enc a b c nc-2 nc-2 nc-2 (pubk d)))
      (send (enc b c d nc-2 nc-2 nc-2 nd-0 (pubk a)))))
  (label 1349)
  (parent 414)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-2 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1350)
  (parent 415)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-2 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1351)
  (parent 415)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (0 1)
    (enc a b c nb-1 nb-0 nc-0 (pubk d))
    (enc a b c nb-1 nb-0 nc-2 (pubk d))
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1352)
  (parent 415)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d)))))
  (label 1353)
  (parent 417)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1354)
  (parent 417)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1355)
  (parent 417)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1356)
  (parent 418)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1357)
  (parent 418)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 2 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1358)
  (parent 418)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1359)
  (parent 418)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1360)
  (parent 418)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1361)
  (parent 419)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a)))))
  (label 1362)
  (parent 419)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1363)
  (parent 419)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1364)
  (parent 420)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1365)
  (parent 420)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1366)
  (parent 420)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1367)
  (parent 420)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c)))))
  (label 1368)
  (parent 420)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1369)
  (parent 421)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1370)
  (parent 421)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1371)
  (parent 421)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1372)
  (parent 421)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1373)
  (parent 422)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1374)
  (parent 422)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1375)
  (parent 422)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d))
    (enc a b c nc nb-0 nc-0 (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b d nc nb-0 (pubk c)))
      (send (enc a b c nc nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 1376)
  (parent 423)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1377)
  (parent 423)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1378)
  (parent 423)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1379)
  (parent 424)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1380)
  (parent 424)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (5 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-1 (pubk a)))))
  (label 1381)
  (parent 424)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1382)
  (parent 425)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 3 resp1 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1383)
  (parent 425)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1384)
  (parent 425)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1385)
  (parent 426)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1386)
  (parent 426)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1387)
  (parent 426)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1388)
  (parent 427)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1389)
  (parent 427)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1390)
  (parent 427)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1391)
  (parent 427)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c)))))
  (label 1392)
  (parent 427)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1393)
  (parent 428)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1394)
  (parent 428)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1395)
  (parent 429)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1396)
  (parent 429)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0) (nd-0 nd))
    nc (0 1) (enc a b c nc nb nc (pubk d))
    (enc a b c nc nb-0 nc-0 (pubk d)) (enc a b d nc nb (pubk c))
    (enc a b d nc nb-0 (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nb-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb-0 nc-0 nd (pubk a)))
      (send (enc nb-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb-0 (pubk c))))
    ((recv (enc a b d nc nb-0 (pubk c)))
      (send (enc a b c nc nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc nb-0 nc-0 (pubk d)))
      (send (enc b c d nc nb-0 nc-0 nd (pubk a)))))
  (label 1397)
  (parent 430)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1398)
  (parent 430)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-1 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nb-1 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1399)
  (parent 430)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc (pubk d))
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))
      (send (enc nb-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))))
  (label 1400)
  (parent 431)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1401)
  (parent 431)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1402)
  (parent 431)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1403)
  (parent 432)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1404)
  (parent 432)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1405)
  (parent 433)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1406)
  (parent 433)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1407)
  (parent 433)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1408)
  (parent 433)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1409)
  (parent 434)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1410)
  (parent 434)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 1411)
  (parent 435)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1412)
  (parent 435)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1413)
  (parent 435)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-0 nb-0) (nc-1 nb-0)) nb-0 (5 0)
    (enc a b c nb-0 nb-0 nb-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 1414)
  (parent 436)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1415)
  (parent 436)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1416)
  (parent 436)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1417)
  (parent 436)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1418)
  (parent 437)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1419)
  (parent 437)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1420)
  (parent 438)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1421)
  (parent 438)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1422)
  (parent 438)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1423)
  (parent 438)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1424)
  (parent 439)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1425)
  (parent 439)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nb nc-0 (pubk d)))
      (send (enc b c d nc nb nc-0 nd (pubk a)))))
  (label 1426)
  (parent 440)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1427)
  (parent 440)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 1428)
  (parent 440)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb nc-0 (pubk d)))
      (send (enc b c d nc-0 nb nc-0 nd (pubk a)))))
  (label 1429)
  (parent 441)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1430)
  (parent 441)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-1 (pubk a)))))
  (label 1431)
  (parent 441)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1432)
  (parent 442)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 4 resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1433)
  (parent 442)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1434)
  (parent 442)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1435)
  (parent 443)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1436)
  (parent 443)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1437)
  (parent 443)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd)) na
    (0 1) (enc a b c na nb nc (pubk d)) (enc a b c na nb nc-0 (pubk d))
    (enc a b d na nb (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb (pubk c))))
    ((recv (enc a b d na nb (pubk c)))
      (send (enc a b c na nb nc-0 (pubk d))))
    ((recv (enc a b c na nb nc-0 (pubk d)))
      (send (enc b c d na nb nc-0 nd (pubk a)))))
  (label 1438)
  (parent 444)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (0 1)
    (enc a b c na nb-0 nc-0 (pubk d)) (enc a b c na nb-0 nc-1 (pubk d))
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b))
    (enc b c d na nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a b c na nb-0 nc-1 (pubk d)))
      (send (enc b c d na nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-1 (pubk a)))))
  (label 1439)
  (parent 444)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1440)
  (parent 445)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d)))))
  (label 1441)
  (parent 445)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1442)
  (parent 445)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1443)
  (parent 445)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1444)
  (parent 445)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1445)
  (parent 445)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc (pubk d))
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))
      (send (enc nb-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))))
  (label 1446)
  (parent 446)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1447)
  (parent 446)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1448)
  (parent 446)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1449)
  (parent 447)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1450)
  (parent 447)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1451)
  (parent 447)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1452)
  (parent 448)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)) (enc b c d na nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1453)
  (parent 448)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1454)
  (parent 449)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1455)
  (parent 449)
  (unrealized (0 1) (2 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-2 (pubk c)))))
  (label 1456)
  (parent 449)
  (unrealized (0 1) (2 0) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1457)
  (parent 449)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1458)
  (parent 449)
  (unrealized (0 1) (2 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1459)
  (parent 449)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1460)
  (parent 450)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1461)
  (parent 450)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1462)
  (parent 450)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1463)
  (parent 450)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1464)
  (parent 451)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1465)
  (parent 451)
  (unrealized (0 1) (2 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0))
    ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-2 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1466)
  (parent 451)
  (unrealized (0 1) (2 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-1 (pubk c))
    (enc a b d nc-1 nb-2 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1467)
  (parent 453)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-1 (pubk c))
    (enc a b d nc-1 nb-2 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c))))
    ((recv (enc a b d nc-1 nb-2 (pubk c)))
      (send (enc a b c nc-1 nb-2 nc-2 (pubk d)))))
  (label 1468)
  (parent 453)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b c nb nb nc-0 (pubk d))
    (enc a b d nb nb (pubk c)) (enc a c d nb (pubk b))
    (enc b c d nb nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc-0 (pubk d))))
    ((recv (enc a b c nb nb nc-0 (pubk d)))
      (send (enc b c d nb nb nc-0 nd (pubk a)))))
  (label 1469)
  (parent 454)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1470)
  (parent 454)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)) (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1471)
  (parent 454)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1472)
  (parent 455)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1473)
  (parent 455)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1474)
  (parent 455)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1475)
  (parent 455)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb nc-0 (pubk d)))
      (send (enc b c d nc-0 nb nc-0 nd (pubk a)))))
  (label 1476)
  (parent 456)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1477)
  (parent 456)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-1 (pubk a)))))
  (label 1478)
  (parent 456)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1479)
  (parent 457)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1480)
  (parent 457)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)) (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1481)
  (parent 457)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1482)
  (parent 458)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1483)
  (parent 458)
  (unrealized (0 1) (2 0) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1484)
  (parent 458)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1485)
  (parent 458)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc (pubk d))
    (enc a b c nc-0 nb nc-0 (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb nc-0 (pubk d)))
      (send (enc b c d nc-0 nb nc-0 nd (pubk a)))))
  (label 1486)
  (parent 459)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1487)
  (parent 459)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc (pubk d))
    (enc a b c nc-0 nb-0 nc-0 (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))
      (send (enc nb-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd (pubk a)))))
  (label 1488)
  (parent 460)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1489)
  (parent 460)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nb-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1490)
  (parent 460)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nb-0 nd text) (a b c d name))
  (defstrand init 3 (na nb) (nb nb-0) (nc nb) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nb) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb) (nb nb-0) (nc nb) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb) (nb nb-0) (nc nb) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb)
  (operation nonce-test (contracted (nb-1 nb-0) (nc-0 nb) (nd-0 nd)) nb
    (0 1) (enc a b c nb nb nc (pubk d)) (enc a b c nb nb-0 nb (pubk d))
    (enc a b d nb nb (pubk c)) (enc a b d nb nb-0 (pubk c))
    (enc a c d nb (pubk b)) (enc b c d nb nb-0 nb nd (pubk a)))
  (traces
    ((send (enc a c d nb (pubk b)))
      (recv (enc b c d nb nb-0 nb nd (pubk a)))
      (send (enc nb-0 nb nd (pubk b))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb (pubk c))))
    ((recv (enc a b d nb nb (pubk c)))
      (send (enc a b c nb nb nc (pubk d))))
    ((recv (enc a c d nb (pubk b))) (send (enc a b d nb nb-0 (pubk c))))
    ((recv (enc a b d nb nb-0 (pubk c)))
      (send (enc a b c nb nb-0 nb (pubk d))))
    ((recv (enc a b c nb nb-0 nb (pubk d)))
      (send (enc b c d nb nb-0 nb nd (pubk a)))))
  (label 1491)
  (parent 461)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nb-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nb-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1492)
  (parent 461)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-1 nb-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b))
    (enc b c d nb-0 nb-1 nb-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nb-0 nd-0 (pubk a))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1493)
  (parent 461)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1494)
  (parent 462)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 3 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1495)
  (parent 462)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c)))))
  (label 1496)
  (parent 462)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-1 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1497)
  (parent 462)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nb-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1498)
  (parent 462)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1499)
  (parent 462)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nb-1) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 7 5 resp3 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-1 (pubk c))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nb-1 nd-0 (pubk d)))
      (send (enc b c d nd-0 nb-1 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nb-1 (pubk c)))
      (send (enc a b c nd-0 nb-1 nc-1 (pubk d)))))
  (label 1500)
  (parent 462)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1501)
  (parent 463)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1502)
  (parent 463)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1503)
  (parent 463)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1504)
  (parent 463)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1505)
  (parent 463)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1506)
  (parent 464)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1507)
  (parent 464)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1508)
  (parent 464)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1509)
  (parent 464)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1510)
  (parent 464)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1511)
  (parent 465)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1512)
  (parent 465)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1513)
  (parent 465)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1514)
  (parent 465)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1515)
  (parent 466)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1516)
  (parent 466)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1517)
  (parent 466)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1518)
  (parent 466)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1519)
  (parent 467)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1520)
  (parent 468)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1521)
  (parent 468)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1522)
  (parent 468)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1523)
  (parent 468)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (4 0)
    (enc a b c nb-0 nb-1 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1524)
  (parent 469)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1525)
  (parent 469)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1526)
  (parent 469)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1527)
  (parent 469)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1528)
  (parent 469)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1529)
  (parent 471)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1530)
  (parent 471)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1531)
  (parent 471)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1532)
  (parent 471)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1533)
  (parent 472)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1534)
  (parent 472)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1535)
  (parent 472)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1536)
  (parent 472)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1537)
  (parent 472)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1538)
  (parent 472)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1539)
  (parent 472)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (6 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1540)
  (parent 472)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1541)
  (parent 473)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1542)
  (parent 473)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1543)
  (parent 473)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1544)
  (parent 473)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1545)
  (parent 473)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1546)
  (parent 473)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1547)
  (parent 473)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (6 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1548)
  (parent 473)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1549)
  (parent 474)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1550)
  (parent 474)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1551)
  (parent 474)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1552)
  (parent 474)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1553)
  (parent 474)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1554)
  (parent 474)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1555)
  (parent 474)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (6 0))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1556)
  (parent 474)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1557)
  (parent 475)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1558)
  (parent 475)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1559)
  (parent 475)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1560)
  (parent 475)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-3 (pubk d)))))
  (label 1561)
  (parent 475)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1562)
  (parent 475)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1563)
  (parent 476)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1564)
  (parent 476)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1565)
  (parent 476)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d)))))
  (label 1566)
  (parent 476)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (2 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-1 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1567)
  (parent 477)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (2 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-1 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1568)
  (parent 477)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1569)
  (parent 477)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d))
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1570)
  (parent 477)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1571)
  (parent 477)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1572)
  (parent 477)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-1 nb-0)) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1573)
  (parent 479)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1574)
  (parent 479)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1575)
  (parent 479)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1576)
  (parent 479)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1577)
  (parent 479)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1578)
  (parent 479)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1579)
  (parent 480)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d))
    (enc a b c nc-1 nb-1 nc-2 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1580)
  (parent 481)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-2 (2 0)
    (enc a b c nc-2 nc-2 nc-2 (pubk d))
    (enc a b c nc-2 nb-1 nc-1 (pubk d)) (enc a b d nc-2 nc-2 (pubk c))
    (enc a b d nc-2 nb-1 (pubk c)) (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1581)
  (parent 481)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d))
    (enc a b c nc-1 nb-1 nc-2 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-3 (pubk d)))))
  (label 1582)
  (parent 481)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d))
    (enc a b c nc-1 nb-1 nc-2 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1583)
  (parent 481)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-1) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d))
    (enc a b c nc-1 nb-1 nc-2 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nc-1 nb-1 nc-2 (pubk d)))
      (send (enc b c d nc-1 nb-1 nc-2 nd-0 (pubk a)))))
  (label 1584)
  (parent 481)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1585)
  (parent 482)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1586)
  (parent 482)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1587)
  (parent 482)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1588)
  (parent 482)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1589)
  (parent 482)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1590)
  (parent 482)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1591)
  (parent 483)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1592)
  (parent 483)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1593)
  (parent 483)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1594)
  (parent 483)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1595)
  (parent 484)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1596)
  (parent 484)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1597)
  (parent 486)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1598)
  (parent 486)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1599)
  (parent 486)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1600)
  (parent 487)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (contracted (nb-1 nb-0)) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1601)
  (parent 488)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (contracted (nb-1 nc-1)) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1602)
  (parent 488)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1603)
  (parent 488)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 1604)
  (parent 488)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1605)
  (parent 488)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1606)
  (parent 488)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1607)
  (parent 489)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1608)
  (parent 489)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 5 resp1 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 1609)
  (parent 489)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1610)
  (parent 489)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a b d na nb-1 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 1611)
  (parent 489)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1612)
  (parent 490)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1613)
  (parent 490)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1614)
  (parent 490)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1615)
  (parent 490)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b c na nb-2 nc-2 (pubk d)))
      (send (enc b c d na nb-2 nc-2 nd-0 (pubk a)))))
  (label 1616)
  (parent 490)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1617)
  (parent 491)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1618)
  (parent 491)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-3 (pubk d)))))
  (label 1619)
  (parent 491)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d)))))
  (label 1620)
  (parent 491)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1621)
  (parent 492)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 1622)
  (parent 492)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1623)
  (parent 492)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp2 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-3 (pubk d)))))
  (label 1624)
  (parent 492)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-2) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp3 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-2 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-2 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1625)
  (parent 492)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 6 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1626)
  (parent 494)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1627)
  (parent 494)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1628)
  (parent 494)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-1 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-2 (pubk d)))
      (send (enc b c d na nb-1 nc-2 nd-0 (pubk a)))))
  (label 1629)
  (parent 494)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-2 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1630)
  (parent 495)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-2 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1631)
  (parent 495)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-2 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1632)
  (parent 495)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-2 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b c na nb-2 nc-2 (pubk d)))
      (send (enc b c d na nb-2 nc-2 nd-0 (pubk a)))))
  (label 1633)
  (parent 495)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1634)
  (parent 496)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1635)
  (parent 496)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1636)
  (parent 496)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1637)
  (parent 496)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1638)
  (parent 496)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d))
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 1639)
  (parent 497)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d))
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1640)
  (parent 497)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-2) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp3 2) nb-2 (2 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d))
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-2 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-2 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1641)
  (parent 497)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-2) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp3 2) nb-2 (2 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d))
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-2 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-2 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1642)
  (parent 497)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-1)) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1643)
  (parent 499)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (contracted (nb-2 nb-0)) na (4 0)
    (enc a b c na nb-1 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-2 (pubk d)))))
  (label 1644)
  (parent 499)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 3 resp1 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1645)
  (parent 499)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1646)
  (parent 499)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 1647)
  (parent 499)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-0 (pubk c))
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b c na nb-2 nc-2 (pubk d)))
      (send (enc b c d na nb-2 nc-2 nd-0 (pubk a)))))
  (label 1648)
  (parent 499)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (displaced 7 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-2 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 1649)
  (parent 500)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-2 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1650)
  (parent 500)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-2 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a)))))
  (label 1651)
  (parent 500)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (2 0)
    (enc a b c na nb-1 nc-1 (pubk d)) (enc a b c na nb-2 nc-2 (pubk d))
    (enc a b d na nb-1 (pubk c)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b c na nb-2 nc-2 (pubk d)))
      (send (enc b c d na nb-2 nc-2 nd-0 (pubk a)))))
  (label 1652)
  (parent 500)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1653)
  (parent 501)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d))
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1654)
  (parent 501)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1655)
  (parent 501)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1656)
  (parent 501)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1657)
  (parent 503)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1658)
  (parent 503)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1659)
  (parent 503)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1660)
  (parent 503)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1661)
  (parent 507)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1662)
  (parent 507)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1663)
  (parent 507)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1664)
  (parent 507)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-2 nd-0 (pubk a)))))
  (label 1665)
  (parent 507)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1666)
  (parent 508)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d))
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1667)
  (parent 508)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1668)
  (parent 508)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1669)
  (parent 508)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1670)
  (parent 510)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1671)
  (parent 512)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1672)
  (parent 512)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 1673)
  (parent 512)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1674)
  (parent 512)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp2 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-3 (pubk d)))))
  (label 1675)
  (parent 512)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-2) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp3 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-0 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-2 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-2 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1676)
  (parent 512)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d))
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d)))))
  (label 1677)
  (parent 513)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d))
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1678)
  (parent 513)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-2) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp3 2) nb-2 (2 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d))
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-2 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-2 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1679)
  (parent 513)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-2) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp3 2) nb-2 (2 0)
    (enc a b c nb-2 nb-1 nc-1 (pubk d))
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-2 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-2 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1680)
  (parent 513)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1681)
  (parent 515)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1682)
  (parent 517)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1683)
  (parent 517)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1684)
  (parent 517)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1685)
  (parent 517)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1686)
  (parent 517)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1687)
  (parent 518)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1688)
  (parent 518)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1689)
  (parent 518)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1690)
  (parent 518)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1691)
  (parent 519)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1692)
  (parent 519)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (3 0)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1693)
  (parent 519)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1694)
  (parent 520)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 1695)
  (parent 520)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c)))))
  (label 1696)
  (parent 520)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 1697)
  (parent 521)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 1698)
  (parent 521)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (4 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1699)
  (parent 522)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 5 2 resp2 2) nc-1 (4 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1700)
  (parent 522)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (0 1))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (4 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-1 (pubk a)))))
  (label 1701)
  (parent 522)
  (unrealized (0 1) (4 0) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (3 0))
    ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (4 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1702)
  (parent 523)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (4 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 1703)
  (parent 523)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (3 0)) ((2 1) (4 0)) ((3 1) (2 0))
    ((4 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb nc) (nc-1 nc-0) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc-0 nd (pubk a)))
      (send (enc nc nc-0 nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc-0 (pubk d)))
      (send (enc b c d nc nc nc-0 nd (pubk a)))))
  (label 1704)
  (parent 524)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((1 1) (3 0)) ((2 1) (4 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 1705)
  (parent 524)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1706)
  (parent 525)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1707)
  (parent 525)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1708)
  (parent 526)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1709)
  (parent 526)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1710)
  (parent 526)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 1711)
  (parent 527)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1712)
  (parent 527)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1713)
  (parent 527)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-0 nb-0) (nc-1 nb-0)) nb-0 (5 0)
    (enc a b c nb-0 nb-0 nb-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 1714)
  (parent 528)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1715)
  (parent 528)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1716)
  (parent 528)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1717)
  (parent 528)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1718)
  (parent 528)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1719)
  (parent 528)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1720)
  (parent 529)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1721)
  (parent 529)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1722)
  (parent 530)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1723)
  (parent 530)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1724)
  (parent 530)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nb nc-0 (pubk d)))
      (send (enc b c d nc nb nc-0 nd (pubk a)))))
  (label 1725)
  (parent 531)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1726)
  (parent 531)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 1727)
  (parent 531)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb nc-0 (pubk d)))
      (send (enc b c d nc-0 nb nc-0 nd (pubk a)))))
  (label 1728)
  (parent 532)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1729)
  (parent 532)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-1 (pubk a)))))
  (label 1730)
  (parent 532)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1731)
  (parent 533)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (2 0)) ((5 1) (4 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1732)
  (parent 533)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (4 0)) ((2 1) (0 1))
    ((3 1) (6 0)) ((4 1) (2 0)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1733)
  (parent 533)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (4 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1734)
  (parent 534)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (4 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1735)
  (parent 534)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (4 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1736)
  (parent 534)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (4 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1737)
  (parent 534)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (4 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1738)
  (parent 536)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (4 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1739)
  (parent 536)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (4 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1740)
  (parent 536)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (4 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1741)
  (parent 536)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1742)
  (parent 537)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (6 0)) ((1 1) (0 1)) ((2 1) (4 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1743)
  (parent 537)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1744)
  (parent 537)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((3 1) (1 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1745)
  (parent 537)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-2) (nb nc-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-2) (nb nc-2) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-2)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-2 (pubk b)))
      (recv (enc b c d nc-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-0 (pubk d))))
    ((recv (enc a c d nc-2 (pubk b)))
      (send (enc a b d nc-2 nc-2 (pubk c))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-1 (pubk d))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-2 (pubk d))))
    ((recv (enc a b c nc-2 nc-2 nc-0 (pubk d)))
      (send (enc b c d nc-2 nc-2 nc-0 nd-0 (pubk a)))))
  (label 1746)
  (parent 537)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1747)
  (parent 538)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (6 0)) ((1 1) (0 1)) ((2 1) (4 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1748)
  (parent 538)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1749)
  (parent 538)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((3 1) (1 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1750)
  (parent 538)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-2) (nb nc-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-2) (nb nc-2) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-2)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-2 (pubk b)))
      (recv (enc b c d nc-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-0 (pubk d))))
    ((recv (enc a c d nc-2 (pubk b)))
      (send (enc a b d nc-2 nc-2 (pubk c))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-1 (pubk d))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-2 (pubk d))))
    ((recv (enc a b c nc-2 nc-2 nc-1 (pubk d)))
      (send (enc b c d nc-2 nc-2 nc-1 nd-0 (pubk a)))))
  (label 1751)
  (parent 538)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a)))))
  (label 1752)
  (parent 539)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (6 0)) ((1 1) (0 1)) ((2 1) (4 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-2 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1753)
  (parent 539)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (5 0)) ((2 1) (4 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-2 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-2 nd-0 (pubk a)))))
  (label 1754)
  (parent 539)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((3 1) (1 0))
    ((3 1) (5 0)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-2 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-2 nd-0 (pubk a)))))
  (label 1755)
  (parent 539)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-2) (nb nc-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-2) (nb nc-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-2) (nb nc-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (4 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-2)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-2 (pubk b)))
      (recv (enc b c d nc-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-0 (pubk d))))
    ((recv (enc a c d nc-2 (pubk b)))
      (send (enc a b d nc-2 nc-2 (pubk c))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-1 (pubk d))))
    ((recv (enc a b d nc-2 nc-2 (pubk c)))
      (send (enc a b c nc-2 nc-2 nc-2 (pubk d))))
    ((recv (enc a b c nc-2 nc-2 nc-2 (pubk d)))
      (send (enc b c d nc-2 nc-2 nc-2 nd-0 (pubk a)))))
  (label 1756)
  (parent 539)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1757)
  (parent 540)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1758)
  (parent 540)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1759)
  (parent 540)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1760)
  (parent 541)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1761)
  (parent 541)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1762)
  (parent 541)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1763)
  (parent 541)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1764)
  (parent 542)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1765)
  (parent 542)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nb nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a b c nc nb nc (pubk d)))
      (send (enc b c d nc nb nc nd (pubk a)))))
  (label 1766)
  (parent 543)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1767)
  (parent 543)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 1768)
  (parent 543)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1769)
  (parent 544)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1770)
  (parent 544)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1771)
  (parent 544)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (5 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-1 (pubk a)))))
  (label 1772)
  (parent 544)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1773)
  (parent 545)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1774)
  (parent 545)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1775)
  (parent 546)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1776)
  (parent 546)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1777)
  (parent 546)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1778)
  (parent 546)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1779)
  (parent 547)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1780)
  (parent 547)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc-0) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc-0) (nd-0 nd)) nc
    (0 1) (enc a b c nc nb nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nb (pubk c)) (enc a b d nc nc (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc-0 nd (pubk a)))
      (send (enc nc nc-0 nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc-0 (pubk d)))
      (send (enc b c d nc nc nc-0 nd (pubk a)))))
  (label 1781)
  (parent 548)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1782)
  (parent 548)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nb-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nb-0 (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1783)
  (parent 548)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nc-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nc-0) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))
      (send (enc nc-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))))
  (label 1784)
  (parent 549)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1785)
  (parent 549)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1786)
  (parent 549)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1787)
  (parent 550)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1788)
  (parent 550)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1789)
  (parent 550)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1790)
  (parent 550)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((3 1) (5 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1791)
  (parent 550)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1792)
  (parent 551)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1793)
  (parent 551)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1794)
  (parent 551)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1795)
  (parent 551)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((3 1) (5 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 3 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1796)
  (parent 551)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 2 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1797)
  (parent 552)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1798)
  (parent 552)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((1 1) (0 1))
    ((2 1) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1799)
  (parent 552)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (5 0)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 1 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1800)
  (parent 552)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1801)
  (parent 553)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1802)
  (parent 553)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1803)
  (parent 553)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-3 (pubk d)))))
  (label 1804)
  (parent 553)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1805)
  (parent 553)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1806)
  (parent 554)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 1807)
  (parent 554)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d)))))
  (label 1808)
  (parent 554)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (5 0)) ((5 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-0)) nb-0 (2 0)
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-1 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1809)
  (parent 555)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 1810)
  (parent 555)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-2 (2 0)
    (enc a b c nb-2 nb-2 nc-1 (pubk d))
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1811)
  (parent 555)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1812)
  (parent 555)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-2 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1813)
  (parent 555)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1814)
  (parent 557)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1815)
  (parent 557)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1816)
  (parent 557)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1817)
  (parent 557)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1818)
  (parent 557)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1819)
  (parent 558)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1820)
  (parent 558)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1821)
  (parent 558)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (2 0)
    (enc a b c nb-1 nb-1 nc-1 (pubk d))
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1822)
  (parent 558)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 5 resp1 2) nc-1 (6 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1823)
  (parent 559)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp1 2) nc-1 (6 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-2 (pubk c)))))
  (label 1824)
  (parent 559)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nc-2 (6 0)
    (enc a b c nc-2 nc-2 nc-2 (pubk d)) (enc a b d nc-2 nc-2 (pubk c))
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d)))))
  (label 1825)
  (parent 559)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (6 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-3 (pubk d)))))
  (label 1826)
  (parent 559)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (6 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1827)
  (parent 559)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0))
    ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1828)
  (parent 560)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1829)
  (parent 561)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1830)
  (parent 561)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1831)
  (parent 561)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (6 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1832)
  (parent 562)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1833)
  (parent 562)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1834)
  (parent 562)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (6 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1835)
  (parent 562)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (contracted (nb-1 nc-1)) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))))
  (label 1836)
  (parent 563)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 1837)
  (parent 563)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1838)
  (parent 563)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nb-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1839)
  (parent 563)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d)))))
  (label 1840)
  (parent 565)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 nb-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1841)
  (parent 565)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1842)
  (parent 565)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1843)
  (parent 565)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nc-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nc-0) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))
      (send (enc nc-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))))
  (label 1844)
  (parent 566)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1845)
  (parent 566)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1846)
  (parent 566)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1847)
  (parent 567)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1848)
  (parent 567)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a b d nb-0 nb-1 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1849)
  (parent 567)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a b d nb-1 nb-0 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c)))))
  (label 1850)
  (parent 567)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (contracted (nb-1 nc-1)) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-0 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c)))))
  (label 1851)
  (parent 569)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1852)
  (parent 569)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1853)
  (parent 569)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1854)
  (parent 569)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (2 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a b d nc-1 nb-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c))))
    ((recv (enc a b d nc-1 nb-1 (pubk c)))
      (send (enc a b c nc-1 nb-1 nc-2 (pubk d)))))
  (label 1855)
  (parent 569)
  (unrealized (0 1) (2 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1856)
  (parent 570)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0)) ((5 1) (2 0))
    ((6 1) (5 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1857)
  (parent 570)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (5 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (2 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1858)
  (parent 570)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nc-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((1 1) (0 1)) ((2 1) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb nc-0) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nc-0 nc (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))
      (send (enc nc-0 nc-0 nd (pubk b))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))))
  (label 1859)
  (parent 571)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp1 2) nc-1 (0 1)
    (enc a b c nc-1 nc-1 nc-0 (pubk d))
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c)))))
  (label 1860)
  (parent 571)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 1)) ((2 1) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nc-1 nc-0 (pubk d))
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)) (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-1 (pubk a)))))
  (label 1861)
  (parent 571)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nc-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nc-0) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc (pubk d))
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))
      (send (enc nc-0 nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd (pubk a)))))
  (label 1862)
  (parent 573)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1863)
  (parent 573)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-0 (pubk d))
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-0 nd-1 (pubk a)))))
  (label 1864)
  (parent 573)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 1 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1865)
  (parent 575)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (7 0)) ((1 1) (0 1))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0))
    ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1866)
  (parent 575)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (6 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (2 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1867)
  (parent 575)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((4 1) (6 0)) ((5 1) (2 0))
    ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-1 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-2 (pubk d)))))
  (label 1868)
  (parent 575)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nd-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nd-0) (nb nd-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nd-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nd-0) (nb nd-0) (nc nd-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nd-0) (nb nd-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 1)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (6 0)) ((6 1) (2 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nd-0)
  (operation nonce-test (displaced 7 5 resp3 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nd-0 (pubk b)))
      (recv (enc b c d nd-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nb-0 (pubk c))))
    ((recv (enc a b d nd-0 nb-0 (pubk c)))
      (send (enc a b c nd-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nd-0 (pubk b)))
      (send (enc a b d nd-0 nd-0 (pubk c))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nd-0 (pubk d))))
    ((recv (enc a b c nd-0 nd-0 nd-0 (pubk d)))
      (send (enc b c d nd-0 nd-0 nd-0 nd-0 (pubk a))))
    ((recv (enc a b d nd-0 nd-0 (pubk c)))
      (send (enc a b c nd-0 nd-0 nc-1 (pubk d)))))
  (label 1869)
  (parent 575)
  (unrealized (0 1) (2 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1870)
  (parent 576)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1871)
  (parent 576)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1872)
  (parent 576)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1873)
  (parent 576)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1874)
  (parent 576)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 1 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1875)
  (parent 577)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1876)
  (parent 577)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1877)
  (parent 577)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1878)
  (parent 577)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1879)
  (parent 577)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (4 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1880)
  (parent 577)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1881)
  (parent 579)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1882)
  (parent 579)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1883)
  (parent 579)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1884)
  (parent 580)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1885)
  (parent 580)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 1886)
  (parent 580)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (6 0)
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1887)
  (parent 580)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1888)
  (parent 581)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1889)
  (parent 581)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1890)
  (parent 582)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-3 (pubk d)))))
  (label 1891)
  (parent 582)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1892)
  (parent 582)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (0 1)
    (enc a b c nb-0 nb-0 nc-0 (pubk d))
    (enc a b c nb-0 nb-0 nc-1 (pubk d))
    (enc a b c nb-0 nb-0 nc-2 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1893)
  (parent 583)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1))
    ((4 1) (3 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nc-0)) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1894)
  (parent 584)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1895)
  (parent 584)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nc-2 (4 0)
    (enc a b c nc-2 nc-2 nc-2 (pubk d))
    (enc a b c nc-2 nc-2 nc-1 (pubk d)) (enc a b d nc-2 nc-2 (pubk c))
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1896)
  (parent 584)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1897)
  (parent 584)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (4 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-2 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-2 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-2 nd-0 (pubk a)))))
  (label 1898)
  (parent 584)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1899)
  (parent 585)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1900)
  (parent 585)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1901)
  (parent 585)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1902)
  (parent 585)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1903)
  (parent 586)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1904)
  (parent 587)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1905)
  (parent 587)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1906)
  (parent 587)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1907)
  (parent 587)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1908)
  (parent 588)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nb) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb-0 nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nb nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a b d nc nb (pubk c))
    (enc a c d nc (pubk b)) (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nb (pubk c))))
    ((recv (enc a b d nc nb (pubk c)))
      (send (enc a b c nc nb nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 1909)
  (parent 589)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1910)
  (parent 589)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nb-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1911)
  (parent 589)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nc-0 nb-0) (nc-1 nb-0)) nb-0 (5 0)
    (enc a b c nb-0 nb-0 nb-0 (pubk d)) (enc a b d nb-0 nb-0 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 1912)
  (parent 590)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1913)
  (parent 590)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1914)
  (parent 590)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1915)
  (parent 590)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (5 0)
    (enc a b c nc-1 nb-0 nc-1 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-1 (pubk a)))))
  (label 1916)
  (parent 590)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((1 1) (5 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 3 resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1917)
  (parent 591)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1918)
  (parent 591)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1919)
  (parent 591)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 6 4 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a)))))
  (label 1920)
  (parent 592)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1921)
  (parent 592)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1922)
  (parent 592)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c)))))
  (label 1923)
  (parent 593)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1924)
  (parent 593)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c)))))
  (label 1925)
  (parent 593)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-2 (5 0)
    (enc a b d nb-2 nb-1 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-1 (pubk c)))))
  (label 1926)
  (parent 593)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1927)
  (parent 594)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 2 resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1928)
  (parent 594)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1929)
  (parent 595)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 6 4 resp2 2) nc-1 (5 0)
    (enc a b c nc-1 nc-1 nc-1 (pubk d)) (enc a b d nc-1 nc-1 (pubk c))
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1930)
  (parent 595)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (5 0)
    (enc a b c nc-0 nc-0 nc-0 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-1 (pubk a)))))
  (label 1931)
  (parent 595)
  (unrealized (0 1) (5 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nb nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nb) (nc nc-0) (nd nd) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (contracted (nb-0 nb) (nc-1 nc-0) (nd-0 nd))
    nc-0 (0 1) (enc a b c nc-0 nb nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc (pubk d)) (enc a b d nc-0 nb (pubk c))
    (enc a b d nc-0 nc-0 (pubk c)) (enc a c d nc-0 (pubk b))
    (enc b c d nc-0 nb nc-0 nd (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc-0 nd (pubk a)))
      (send (enc nb nc-0 nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb (pubk c))))
    ((recv (enc a b d nc-0 nb (pubk c)))
      (send (enc a b c nc-0 nb nc-0 (pubk d))))
    ((recv (enc a b c nc-0 nb nc-0 (pubk d)))
      (send (enc b c d nc-0 nb nc-0 nd (pubk a)))))
  (label 1932)
  (parent 596)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp2 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-2 (pubk d)))))
  (label 1933)
  (parent 596)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (added-strand resp3 2) nc-1 (0 1)
    (enc a b c nc-1 nb-0 nc-1 (pubk d))
    (enc a b c nc-1 nc-1 nc-0 (pubk d)) (enc a b d nc-1 nb-0 (pubk c))
    (enc a b d nc-1 nc-1 (pubk c)) (enc a c d nc-1 (pubk b))
    (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b d nc-1 nb-0 (pubk c)))
      (send (enc a b c nc-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nb-0 nc-1 (pubk d)))
      (send (enc b c d nc-1 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-1 (pubk a)))))
  (label 1934)
  (parent 596)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1935)
  (parent 597)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1936)
  (parent 597)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1937)
  (parent 597)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1938)
  (parent 597)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1939)
  (parent 597)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1940)
  (parent 597)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1941)
  (parent 598)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1942)
  (parent 598)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1943)
  (parent 598)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1944)
  (parent 598)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1945)
  (parent 598)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1946)
  (parent 598)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1947)
  (parent 599)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1948)
  (parent 599)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a)))))
  (label 1949)
  (parent 599)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-1 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1950)
  (parent 599)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1951)
  (parent 599)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1952)
  (parent 600)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1953)
  (parent 600)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 1954)
  (parent 600)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-2 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1955)
  (parent 600)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (0 1))
    ((4 1) (0 1)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1956)
  (parent 600)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (contracted (nb-2 nb-1)) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1957)
  (parent 601)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1958)
  (parent 601)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nb-2 (6 0)
    (enc a b d nb-2 nb-0 (pubk c)) (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1959)
  (parent 601)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-3 (pubk d)))))
  (label 1960)
  (parent 601)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d)))))
  (label 1961)
  (parent 602)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1962)
  (parent 602)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-2 (4 0)
    (enc a b c nb-2 nb-1 nc-2 (pubk d)) (enc a b d nb-2 nb-1 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1963)
  (parent 602)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp3 2) nb-0 (4 0)
    (enc a b c nb-0 nb-2 nc-2 (pubk d)) (enc a b d nb-0 nb-2 (pubk c))
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-0 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-0 nb-2 nc-2 nd-0 (pubk a)))))
  (label 1964)
  (parent 602)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1965)
  (parent 604)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1966)
  (parent 604)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1967)
  (parent 604)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nc-3 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 1968)
  (parent 604)
  (unrealized (0 1) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d)))))
  (label 1969)
  (parent 605)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c)))))
  (label 1970)
  (parent 605)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1971)
  (parent 605)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1972)
  (parent 605)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp3 2) nb-1 (4 0)
    (enc a b c nb-1 nb-1 nc-2 (pubk d)) (enc a b d nb-1 nb-1 (pubk c))
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-2 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-2 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-2 nd-0 (pubk a)))))
  (label 1973)
  (parent 605)
  (unrealized (0 1) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp2 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1974)
  (parent 606)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1975)
  (parent 606)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a b d nc-0 nb-0 (pubk c)) (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-0 (pubk a)))))
  (label 1976)
  (parent 606)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-2 (pubk d)))))
  (label 1977)
  (parent 608)
  (unrealized (0 1) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d)))))
  (label 1978)
  (parent 608)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1979)
  (parent 609)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d)))))
  (label 1980)
  (parent 609)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b d nc-0 nb-1 (pubk c)))
      (send (enc a b c nc-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-2 (pubk c)))))
  (label 1981)
  (parent 609)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-2 (6 0)
    (enc a c d nc-2 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d)))))
  (label 1982)
  (parent 609)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 1 resp1 2) nc-0 (6 0)
    (enc a c d nc-0 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-1 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1983)
  (parent 610)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nb-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nb-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (4 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nb-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nb-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nb-0 nd-0 (pubk a)))))
  (label 1984)
  (parent 610)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nb-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nb-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (4 0)) ((3 1) (0 1)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (displaced 7 5 resp1 2) nc-1 (6 0)
    (enc a c d nc-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nb-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nb-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nb-1 nd-0 (pubk a)))))
  (label 1985)
  (parent 610)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1986)
  (parent 611)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1987)
  (parent 611)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nc-2 nb-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c)))))
  (label 1988)
  (parent 611)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d)))))
  (label 1989)
  (parent 611)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nc-2 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b d nc-0 nb-0 (pubk c)))
      (send (enc a b c nc-0 nb-0 nc-2 (pubk d)))))
  (label 1990)
  (parent 611)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 1 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1991)
  (parent 612)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nb-1 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 3 resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1992)
  (parent 612)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (6 0))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (displaced 7 5 resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a)))))
  (label 1993)
  (parent 612)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (6 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))))
  (label 1994)
  (parent 612)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nb-0 nd-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (6 0)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a)))))
  (label 1995)
  (parent 612)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nb-0 nc-1 nd-0 nb-1 nc-2 text) (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (0 1))
    ((5 1) (0 1)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-1 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-1 (pubk d))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-2 (pubk d)))))
  (label 1996)
  (parent 613)
  (unrealized (0 1) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (displaced 7 2 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nb-0 (pubk c)))))
  (label 1997)
  (parent 613)
  (unrealized (0 1))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nb-0 text) (a b c d name))
  (defstrand init 3 (na nc-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-1) (nb nc-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-1) (nb nc-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-1) (nb nc-1) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nc-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (5 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-1)
  (operation nonce-test (displaced 7 4 resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-0 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nc-1 (pubk b)))
      (recv (enc b c d nc-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-0 (pubk d))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nc-1 (pubk c))))
    ((recv (enc a b d nc-1 nc-1 (pubk c)))
      (send (enc a b c nc-1 nc-1 nc-1 (pubk d))))
    ((recv (enc a b c nc-1 nc-1 nc-0 (pubk d)))
      (send (enc b c d nc-1 nc-1 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nc-1 (pubk b)))
      (send (enc a b d nc-1 nb-0 (pubk c)))))
  (label 1998)
  (parent 613)
  (unrealized (0 1) (5 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nc nc-0 nd text) (a b c d name))
  (defstrand init 3 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc) (a a) (b b) (c c) (d d))
  (defstrand resp1 2 (na nc) (nb nc) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc) (nb nc) (nc nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nc) (nb nc) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (2 0)) ((2 1) (5 0))
    ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc)
  (operation nonce-test (contracted (nb nc) (nc-1 nc) (nd-0 nd)) nc
    (0 1) (enc a b c nc nc nc (pubk d)) (enc a b c nc nc nc-0 (pubk d))
    (enc a b d nc nc (pubk c)) (enc a c d nc (pubk b))
    (enc b c d nc nc nc nd (pubk a)))
  (traces
    ((send (enc a c d nc (pubk b)))
      (recv (enc b c d nc nc nc nd (pubk a)))
      (send (enc nc nc nd (pubk b))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc (pubk d))))
    ((recv (enc a c d nc (pubk b))) (send (enc a b d nc nc (pubk c))))
    ((recv (enc a b d nc nc (pubk c)))
      (send (enc a b c nc nc nc-0 (pubk d))))
    ((recv (enc a b c nc nc nc (pubk d)))
      (send (enc b c d nc nc nc nd (pubk a)))))
  (label 1999)
  (parent 614)
  (unrealized)
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nc-0 nc-1 nd-0 nd-1 text) (a b c d name))
  (defstrand init 3 (na nc-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nc-0) (nb nc-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nc-0) (nb nc-0) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp3 2 (na nc-0) (nb nc-0) (nc nc-1) (nd nd-1) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((1 1) (2 0))
    ((2 1) (5 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (0 1))
    ((6 1) (0 1)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nc-0)
  (operation nonce-test (added-strand resp3 2) nc-0 (0 1)
    (enc a b c nc-0 nc-0 nc-0 (pubk d))
    (enc a b c nc-0 nc-0 nc-1 (pubk d)) (enc a b d nc-0 nc-0 (pubk c))
    (enc a c d nc-0 (pubk b)) (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a)))
  (traces
    ((send (enc a c d nc-0 (pubk b)))
      (recv (enc b c d nc-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-0 (pubk d))))
    ((recv (enc a c d nc-0 (pubk b)))
      (send (enc a b d nc-0 nc-0 (pubk c))))
    ((recv (enc a b d nc-0 nc-0 (pubk c)))
      (send (enc a b c nc-0 nc-0 nc-1 (pubk d))))
    ((recv (enc a b c nc-0 nc-0 nc-0 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a b c nc-0 nc-0 nc-1 (pubk d)))
      (send (enc b c d nc-0 nc-0 nc-1 nd-1 (pubk a)))))
  (label 2000)
  (parent 614)
  (unrealized (0 1) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp1 2) na (6 0)
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b)))
      (send (enc a b d na nb-3 (pubk c)))))
  (label 156)
  (parent 54)
  (unrealized (0 1) (2 0) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nd-0 nb-1 nc-1 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a b c na nb-0 nc-0 (pubk d)))
      (send (enc b c d na nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 239)
  (parent 85)
  (unrealized (0 1) (3 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 313)
  (parent 107)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nd-0 nb-2 nc-1 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-0) (nb nb-0) (nc nc-0) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b c nb-0 nb-0 nc-0 (pubk d)))
      (send (enc b c d nb-0 nb-0 nc-0 nd-0 (pubk a))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 356)
  (parent 118)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-2 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-3 (pubk d)))))
  (label 410)
  (parent 132)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (5 0)
    (enc a b d na nb-2 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a b c na nb-1 nc-1 (pubk d)))
      (send (enc b c d na nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d)))))
  (label 452)
  (parent 141)
  (unrealized (0 1) (2 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 470)
  (parent 148)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 478)
  (parent 151)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nd-0 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 485)
  (parent 152)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-0 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-3 (pubk d)))))
  (label 493)
  (parent 153)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp2 2) na (6 0)
    (enc a b d na nb-1 (pubk c)) (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-3 (pubk d)))))
  (label 498)
  (parent 154)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (na nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na na) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na na) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na na) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na na) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig na)
  (operation nonce-test (added-strand resp3 2) na (4 0)
    (enc a b c na nb-2 nc-2 (pubk d)) (enc a b d na nb-2 (pubk c))
    (enc a c d na (pubk b)))
  (traces
    ((send (enc a c d na (pubk b)))
      (recv (enc b c d na nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-0 (pubk c))))
    ((recv (enc a b d na nb-0 (pubk c)))
      (send (enc a b c na nb-0 nc-0 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-1 (pubk c))))
    ((recv (enc a b d na nb-1 (pubk c)))
      (send (enc a b c na nb-1 nc-1 (pubk d))))
    ((recv (enc a c d na (pubk b))) (send (enc a b d na nb-2 (pubk c))))
    ((recv (enc a b d na nb-2 (pubk c)))
      (send (enc a b c na nb-2 nc-2 (pubk d))))
    ((recv (enc a b c na nb-2 nc-2 (pubk d)))
      (send (enc b c d na nb-2 nc-2 nd-0 (pubk a)))))
  (label 502)
  (parent 155)
  (unrealized (0 1) (2 0) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 504)
  (parent 157)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (6 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (6 0)
    (enc a b d nb-0 nb-0 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-3 (pubk d)))))
  (label 506)
  (parent 157)
  (unrealized (0 1) (4 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 509)
  (parent 158)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (6 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (6 0)
    (enc a b d nb-1 nb-1 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-3 (pubk d)))))
  (label 511)
  (parent 158)
  (unrealized (0 1) (2 0) (6 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp1 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-3 (pubk c)))))
  (label 514)
  (parent 159)
  (unrealized (0 1) (2 0) (4 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nd-0 text)
    (a b c d name))
  (defstrand init 3 (na nb-2) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-2) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-2) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-2) (nb nb-2) (nc nc-2) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-2)
  (operation nonce-test (added-strand resp3 2) nb-2 (4 0)
    (enc a b c nb-2 nb-2 nc-2 (pubk d)) (enc a b d nb-2 nb-2 (pubk c))
    (enc a c d nb-2 (pubk b)))
  (traces
    ((send (enc a c d nb-2 (pubk b)))
      (recv (enc b c d nb-2 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-0 (pubk c))))
    ((recv (enc a b d nb-2 nb-0 (pubk c)))
      (send (enc a b c nb-2 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-1 (pubk c))))
    ((recv (enc a b d nb-2 nb-1 (pubk c)))
      (send (enc a b c nb-2 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-2 (pubk b)))
      (send (enc a b d nb-2 nb-2 (pubk c))))
    ((recv (enc a b d nb-2 nb-2 (pubk c)))
      (send (enc a b c nb-2 nb-2 nc-2 (pubk d))))
    ((recv (enc a b c nb-2 nb-2 nc-2 (pubk d)))
      (send (enc b c d nb-2 nb-2 nc-2 nd-0 (pubk a)))))
  (label 516)
  (parent 159)
  (unrealized (0 1) (2 0) (4 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nc-2 nb-2 nc-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-3) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (4 0)) ((2 1) (0 1)) ((3 1) (2 0)) ((4 1) (2 0))
    ((5 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp2 2) nb-0 (5 0)
    (enc a b d nb-0 nb-2 (pubk c)) (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-3 (pubk d)))))
  (label 535)
  (parent 166)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (2 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp1 2) nb-1 (6 0)
    (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-3 (pubk c)))))
  (label 556)
  (parent 172)
  (unrealized (0 1) (2 0) (6 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nd-0 nb-2 nc-2 text)
    (a b c d name))
  (defstrand init 3 (na nb-1) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp3 2 (na nb-1) (nb nb-1) (nc nc-1) (nd nd-0) (a a) (b b)
    (c c) (d d))
  (defstrand resp1 2 (na nb-1) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-1) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (6 0)) ((0 0) (7 0))
    ((1 1) (0 1)) ((2 1) (0 1)) ((3 1) (4 0)) ((4 1) (2 0))
    ((5 1) (2 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-1)
  (operation nonce-test (added-strand resp2 2) nb-1 (5 0)
    (enc a b d nb-1 nb-2 (pubk c)) (enc a c d nb-1 (pubk b)))
  (traces
    ((send (enc a c d nb-1 (pubk b)))
      (recv (enc b c d nb-1 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-0 (pubk c))))
    ((recv (enc a b d nb-1 nb-0 (pubk c)))
      (send (enc a b c nb-1 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-1 (pubk c))))
    ((recv (enc a b d nb-1 nb-1 (pubk c)))
      (send (enc a b c nb-1 nb-1 nc-1 (pubk d))))
    ((recv (enc a b c nb-1 nb-1 nc-1 (pubk d)))
      (send (enc b c d nb-1 nb-1 nc-1 nd-0 (pubk a))))
    ((recv (enc a c d nb-1 (pubk b)))
      (send (enc a b d nb-1 nb-2 (pubk c))))
    ((recv (enc a b d nb-1 nb-2 (pubk c)))
      (send (enc a b c nb-1 nb-2 nc-2 (pubk d)))))
  (label 568)
  (parent 175)
  (unrealized (0 1) (2 0) (5 0) (7 0))
  (comment "aborted"))

(defskeleton nsl4cm
  (vars (nb nc nd nb-0 nc-0 nb-1 nc-1 nb-2 nc-2 nb-3 text)
    (a b c d name))
  (defstrand init 3 (na nb-0) (nb nb) (nc nc) (nd nd) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-0) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-0) (nc nc-0) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-1) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-1) (nc nc-1) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-2) (a a) (b b) (c c) (d d))
  (defstrand resp2 2 (na nb-0) (nb nb-2) (nc nc-2) (a a) (b b) (c c)
    (d d))
  (defstrand resp1 2 (na nb-0) (nb nb-3) (a a) (b b) (c c) (d d))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 0) (5 0)) ((0 0) (7 0))
    ((1 1) (2 0)) ((2 1) (0 1)) ((3 1) (0 1)) ((4 1) (0 1))
    ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk a) (privk b) (privk c) (privk d))
  (uniq-orig nb-0)
  (operation nonce-test (added-strand resp1 2) nb-0 (6 0)
    (enc a c d nb-0 (pubk b)))
  (traces
    ((send (enc a c d nb-0 (pubk b)))
      (recv (enc b c d nb-0 nb nc nd (pubk a)))
      (send (enc nb nc nd (pubk b))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-0 (pubk c))))
    ((recv (enc a b d nb-0 nb-0 (pubk c)))
      (send (enc a b c nb-0 nb-0 nc-0 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-1 (pubk c))))
    ((recv (enc a b d nb-0 nb-1 (pubk c)))
      (send (enc a b c nb-0 nb-1 nc-1 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-2 (pubk c))))
    ((recv (enc a b d nb-0 nb-2 (pubk c)))
      (send (enc a b c nb-0 nb-2 nc-2 (pubk d))))
    ((recv (enc a c d nb-0 (pubk b)))
      (send (enc a b d nb-0 nb-3 (pubk c)))))
  (label 603)
  (parent 184)
  (unrealized (0 1) (4 0) (6 0))
  (comment "aborted"))
